@@ -3,6 +3,7 @@ from __future__ import division
 from __future__ import print_function
 import gc
 import shutil
+import builtins
 import subprocess
 import functools
 import os
@@ -37,13 +38,14 @@ from torch._six import container_abcs
 from torch.nn.parameter import Parameter
 from trident.backend.tensorspec import *
 from trident.backend.common import to_list, addindent, camel2snake, unpack_singleton, enforce_singleton, OrderedDict, get_session, set_session, get_session_value, \
-    PrintException, Signature, TensorShape, split_path, make_dir_if_need, sanitize_path
+    PrintException, Signature, TensorShape, split_path, make_dir_if_need, sanitize_path, get_args_spec
 from trident.backend.tensorspec import *
 from trident.backend import iteration_tools
 from trident.backend.pytorch_ops import *
 from trident.backend import pytorch_ops as tops
 from trident.backend.common import dtype as Dtype
 from trident import context
+
 ctx = context._context()
 _backend = ctx.get_backend()
 
@@ -80,7 +82,7 @@ def get_device():
     Returns: device string ('cpu', 'cuda)
 
     """
-    if get_session().device is None:
+    if ctx.device is None:
         set_device("cuda" if torch.cuda.is_available() else "cpu")
     return get_session().device
 
@@ -96,12 +98,12 @@ def set_device(device='cpu'):
             for i in range(len(gcitems)):
                 obj = gcitems[i]
                 try:
-                    if is_tensor(obj) or hasattr(obj, 'data'):
+                    if is_tensor(obj) :
                         obj.to(device)
                     elif isinstance(obj, nn.Module):
                         obj.to(device)
-                except Exception:
-                    pass
+                except Exception as e:
+                    print(e)
     except Exception as e:
         print(e)
 
@@ -341,7 +343,8 @@ class Layer(nn.Module):
 
         default_name: default_name is the same concept as in keras, it comes from class name with sequence number.
 
-        relative_name:relative_name is the same concept as named_modules in pytorch. But in pytorch, you need to get the name from generator enumeration. In trident, you can access the relative name  with this attribute.
+        relative_name:relative_name is the same concept as named_modules in pytorch. But in pytorch, you need to get the name from generator enumeration. In trident,
+        you can access the relative name  with this attribute.
 
 
 
@@ -381,10 +384,9 @@ class Layer(nn.Module):
         self._output_shape: Optional[None, TensorShape, List[TensorShape]] = None
 
         self.input_filters = None
-        self.input_spec = None
 
         self.keep_output = keep_output
-        self.register_buffer('_output_tensor', None, False)
+        self.register_buffer('_output_tensor', None,persistent= False)
 
         self._signature = None
         self._device = get_device()
@@ -413,10 +415,10 @@ class Layer(nn.Module):
         if not hasattr(self, '_nodes') or self._nodes is None:
             self.is_root = True
             return self
-        elif len(self._nodes) > 0 and self._nodes.value_list[0].is_root:
-            return self._nodes.value_list[0]
+        elif len(self._nodes) > 0 and list(self._nodes.values())[0].is_root:
+            return list(self._nodes.values())[0]
         else:
-            for name, node in self._nodes.item_list:
+            for name, node in self._nodes.items():
                 if node.is_root:
                     return node
             return self
@@ -527,7 +529,7 @@ class Layer(nn.Module):
         """
         pass
 
-    def rebuild(self, *input_shape):
+    def rebuild(self, input_shape):
         """ Do the shape inference and initialize weights and bias.
 
         `build' is a key method in trident, you can use  property `built' to check whether the layer do the build process.
@@ -541,12 +543,17 @@ class Layer(nn.Module):
         ans = input('(Y/N) << ').lower()
         if ans in ['yes', 'y']:
             for name, module in self.named_modules():
-                if module.trainable == True:
+                if len(module._parameters) == 0 or module.trainable:
                     module._input_shape = None
                     module._output_shape = None
                     module._built = False
-                    module._parameters = OrderedDict()
-            dummay_input = to_tensor(np.random.standard_normal((1,) + tuple(input_shape)).astype(np.float32)).to(get_device())
+                    for k in module._parameters.keys():
+                        module._parameters[k] = None
+
+                if len(module._forward_pre_hooks) > 0:
+                    module._forward_pre_hooks = OrderedDict()
+
+            dummay_input = to_tensor(np.random.standard_normal((2,) + input_shape).astype(np.float32)).to(get_device())
             out = self.forward(dummay_input)
 
     @property
@@ -652,19 +659,13 @@ class Layer(nn.Module):
             self.to(value)
 
     def cuda(self, device=None):
-        if self.get_root().device != 'cuda':
-            self.get_root().device = 'cuda'
-            super().cuda(device=device)
+        return self._apply(lambda t: t.cuda())
 
     def cpu(self):
-        if self.get_root().device != 'cpu':
-            self.get_root().device = 'cpu'
-            super().cpu()
+        return self._apply(lambda t: t.cpu())
 
     def gpu(self, device=None):
-        if self.get_root().device != 'cuda':
-            self.get_root().device = 'cuda'
-            super().cuda(device)
+        return self._apply(lambda t: t.cuda())
 
     def to(self, *args, **kwargs):
         r"""Moves and/or casts the parameters and buffers.
@@ -767,12 +768,12 @@ class Layer(nn.Module):
         """ Setting the input_shape, means the layer get shape information and start to do the shape inferrence """
 
         if is_tensor(value) and value.ndim == 1 and value.dtype == torch.int32:
-            value = TensorShape( [None,]+to_list(to_numpy(value)))
+            value = TensorShape([None, ] + to_list(to_numpy(value)))
         elif isinstance(value, torch.Size):
-            dims = [None,]+[d for d in value]
+            dims = [None, ] + [d for d in value]
             value = TensorShape(dims)
         elif isinstance(value, (list, tuple)) and len(value) > 0 and all([isinstance(item, numbers.Integral) for item in value]):
-            value = TensorShape((None,)+value)
+            value = TensorShape((None,) + value)
         elif isinstance(value, (list, tuple)) and len(value) > 0 and all([is_tensor(item) and ndim(item) == 1 and item.dtype == torch.int32 for item in value]):
             value = [TensorShape(sh) for sh in value]
         elif isinstance(value, TensorShape):
@@ -795,6 +796,11 @@ class Layer(nn.Module):
                     for k in range(len(self._input_shape)):
                         self._signature.inputs['input_{0}'.format(k)] = TensorSpec(shape=self._input_shape[k], name='input_{0}'.format(k))
 
+    @property
+    def input_spec(self):
+        if self.is_root and self.signature is not None:
+            return unpack_singleton(self.signature.inputs.value_list)
+
     @property
     def output_shape(self):
         return self._output_shape
@@ -802,7 +808,7 @@ class Layer(nn.Module):
     @output_shape.setter
     def output_shape(self, value):
         if is_tensor(value) and value.ndim == 1 and value.dtype == torch.int32:
-            value = TensorShape( [None,]+to_list(to_numpy(value)))
+            value = TensorShape([None, ] + to_list(to_numpy(value)))
         elif isinstance(value, torch.Size):
             dims = [None, ] + [d for d in value]
             value = TensorShape(dims)
@@ -945,19 +951,21 @@ class Layer(nn.Module):
         if not self._built:
             inp = unpack_singleton(input)
             if is_tensor(inp):
-                shp = tensor_to_shape(inp,need_exclude_batch_axis=True)
+                shp = tensor_to_shape(inp, need_exclude_batch_axis=True)
                 self.input_filters = shp[self.filter_index]
                 self.input_shape = shp
-                self.input_spec = TensorSpec.tensor_to_spec(inp)
+                if self.is_root:
+                    self.signature.inputs['input'] = TensorSpec.tensor_to_spec(inp, need_exclude_batch_axis=True, is_singleton=False)
                 del inp
-            elif isinstance(input, (tuple, list)):
-                if isinstance(input[0], numbers.Number):
-                    self.input_shape = TensorShape(list(input))
+            elif isinstance(inp, (tuple, list)):
+                if isinstance(inp[0], numbers.Number):
+                    self.input_shape = TensorShape(list(inp))
                 else:
-                    self.build(*[tensor_to_shape(inp, need_exclude_batch_axis=True) for inp in input])
-
+                    shp=tensor_to_shape(inp[0], need_exclude_batch_axis=True)
+                    self.input_filters = shp[self.filter_index]
+                    self.build(shp)
             else:
-                self.input_shape = TensorShape(list(input))
+                self.input_shape = TensorShape(list(inp))
                 print('input shou be tensor or tuple of tensor')
 
             self._built = True
@@ -966,12 +974,13 @@ class Layer(nn.Module):
             result = self._slow_forward(*input, **kwargs)
         else:
             result = self.forward(*input, **kwargs)
+            result=unpack_singleton(result)
 
             if hasattr(self, 'keep_output') and self.keep_output == True:
                 # make a op
-                self._output_tensor = identity(unpack_singleton(result))
+                self._output_tensor =result
             if self._output_shape is None or is_built == False:
-                output = unpack_singleton(result)
+                output = result
                 if is_tensor(output):  # one output
                     self._output_shape = tensor_to_shape(output)
                 elif isinstance(output, (list, tuple)):
@@ -1142,12 +1151,13 @@ class Sequential(Layer):
         super(Sequential, self).__init__()
         self._name = name
         self._built = False
-        if len(args) == 1 and isinstance(args[0], OrderedDict):
-            for key, module in args[0].items():
+        args = unpack_singleton(args)
+        if isinstance(args, (dict, OrderedDict, ModuleDict, nn.ModuleDict)):
+            for key, module in args.items():
                 module.name = key
                 self.add_module(key, module)
-        elif len(args) == 1 and isinstance(args[0], (list, nn.ModuleList)):
-            for idx, module in enumerate(args[0]):
+        elif isinstance(args, (list, tuple, nn.ModuleList, ModuleList)):
+            for idx, module in enumerate(args):
                 self.add_module(str(idx), module)
         else:
             for idx, module in enumerate(args):
@@ -1187,7 +1197,7 @@ class Sequential(Layer):
             out = module(dummay_input)
             self._modules[name] = module
             self._output_shape = tensor_to_shape(out, need_exclude_batch_axis=True, is_singleton=False)
-            self.get_root().signature.outputs.value_list[0].shape=self._output_shape.copy()
+            self.get_root().signature.outputs.value_list[0].shape = self._output_shape.copy()
         else:
             super(Sequential, self).add_module(name, module)
 
@@ -1258,8 +1268,21 @@ class Sequential(Layer):
 
     def forward(self, x, **kwargs):
         for module in self._modules.values():
-            x = enforce_singleton(x)
-            x = module(x)
+            #x = enforce_singleton(x)
+            if isinstance(x, tuple):
+                arg_spec=get_args_spec(module.forward)
+                if len(arg_spec.args)==1:
+                    x=enforce_singleton(x)
+                x = module(x, **kwargs)
+
+            else:
+                x = module(x, **kwargs)
+            # class_name=module.__class__.__name__.lower()
+            # if 'lstm' in class_name or 'gru' in class_name:
+            #     if isinstance(x,tuple):
+            #         x
+            #         kwargs['hx']=hx
+
         return x
 
 
@@ -1287,8 +1310,9 @@ class ModuleList(Layer):
                 return x
     """
 
-    def __init__(self, modules=None, name=None):
-        super(ModuleList, self).__init__()
+    def __init__(self, modules=None, name=None, keep_output=False):
+        super(ModuleList, self).__init__(name=name, keep_output=keep_output)
+        self.uuid = uuid.uuid4().node
         self._name = name
         if isinstance(modules, dict):
             for key, value in modules.items():
@@ -1421,6 +1445,7 @@ class ModuleDict(Layer):
 
     def __init__(self, modules: Optional[Mapping[str, Layer]] = None, name=None, keep_output=False, is_multicasting=False, **kwargs) -> None:
         super(ModuleDict, self).__init__(name=None, keep_output=False, **kwargs)
+        self.uuid = uuid.uuid4().node
         self.is_multicasting = is_multicasting
         if modules is not None:
             if len(modules) > 0:
@@ -1538,11 +1563,12 @@ class ModuleDict(Layer):
                 self._built = True
 
     def forward(self, x, **kwargs):
-        if self.is_multicasting == True:
-            x = enforce_singleton(x)
+        if self.is_multicasting:
+
+            #x = enforce_singleton(x)
             results = OrderedDict()
             for name, module in self.items():
-                out = module(x)
+                out = module(x, **kwargs)
                 results[name] = out
             return results
         else:
@@ -1640,7 +1666,7 @@ def print_network(net, verbose=False):
     logging.info('Total number of parameters: %d\n' % num_params)
 
 
-def summary(model, input_size, batch_size=1, device="cuda"):
+def summary(model, input_specs, batch_size=1, device="cuda"):
     def register_hook(module):
         def hook(module, input, output):
             # class_name =module.re    module.name   # str(module.__class__).split(".")[-1].split("'")[0]
@@ -1672,18 +1698,25 @@ def summary(model, input_size, batch_size=1, device="cuda"):
             params = 0
             summary[m_key]["flops"] = np.array([0], dtype=np.float64)
             summary[m_key]["macc"] = np.array([0], dtype=np.float64)
-            if hasattr(module, "weight") and hasattr(module.weight, "size"):
-                params += torch.prod(torch.LongTensor(list(module.weight.shape)))
-                summary[m_key]["weight"] = list(module.weight.shape)
-                summary[m_key]["trainable"] = module.weight.requires_grad
-                summary[m_key]["flops"] += (2 * np.prod(np.array(summary[m_key]["weight"]).astype(np.float64)) - 1) * np.prod(
-                    np.array(summary[m_key]["output_shape"][2:]).astype(np.float64))
-                summary[m_key]["macc"] += np.prod(np.array(summary[m_key]["weight"]).astype(np.float64)) * np.prod(np.array(summary[m_key]["output_shape"][2:]).astype(np.float64))
+            summary[m_key]["trainable"] = np.array([0], dtype=np.float64)
+            summary[m_key]["weight"] = OrderedDict()
+            summary[m_key]["bias"] = OrderedDict()
+            for name, para in module._parameters.items():
+                if para is not None:
+                    para_type = "weight"
+                    if 'bias' in name or 'beta' in name:
+                        para_type = "bias"
+
+                    summary[m_key][para_type][name] = list(int_shape(para))
+                    num_params = np.prod(np.array(list(int_shape(para)), dtype=np.float64))
+                    spatial_dims = np.prod(np.array(summary[m_key]["output_shape"][2:]).astype(np.float64))
+                    params += num_params
+                    if para.requires_grad:
+                        summary[m_key]["trainable"] += num_params
+
+                    summary[m_key]["flops"] += (2 * num_params - 1) * spatial_dims
+                    summary[m_key]["macc"] += num_params * spatial_dims
 
-            if hasattr(module, "bias") and module.bias is not None and hasattr(module.bias, "size"):
-                params += torch.prod(torch.LongTensor(list(module.bias.shape)))
-                summary[m_key]["bias"] = list(module.bias.shape)
-                summary[m_key]["flops"] += np.prod(np.array(summary[m_key]["bias"]).astype(np.float64)) * np.prod(np.array(summary[m_key]["output_shape"][2:]).astype(np.float64))
             summary[m_key]["nb_params"] = params
 
         if (
@@ -1712,7 +1745,7 @@ def summary(model, input_size, batch_size=1, device="cuda"):
     model.eval()
 
     # batch_size of 2 for batchnorm
-    x = [to_tensor(shape.get_dummy_tensor()).to(get_device()) for shape in input_size]
+    x = [to_tensor(spec.shape.get_dummy_tensor(), dtype=spec.dtype).to(get_device()) for spec in input_specs]
     # p    rint(type(x[0]))
 
     # create properties
@@ -1729,16 +1762,27 @@ def summary(model, input_size, batch_size=1, device="cuda"):
     # remove these hooks
     for h in hooks:
         h.remove()
+    max_name_len = 0
+    max_weight_len = 0
+    for layer in summary:
+        max_name_len = builtins.max(max_name_len, len(layer + "  [" + summary[layer]["class_name"] + "]") + 5)
+        max_weight_len = builtins.max(max_weight_len, builtins.max([len(str(item).replace('(', '').replace(')', '')) for item in summary[layer]["weight"].items()]) + 5 if len(
+            summary[layer]["weight"]) > 0 else 5)
 
     print("--------------------------------------------------------------------------------------------------------------------------------")
-    line_new = "{0:^50s} {1:<25s}  {2:<20s} {3:<8s}  {4:<8s}  {5:<25s}".format("Layer (type)", "Output Shape", "Weight ", "Bias", "Param #", "FLOPS #")
+    line_new = "{0:^50s} {1:<25s}  {2:<35s} {3:<8s}  {4:<8s}  {5:<25s}".replace('50s', str(max_name_len) + 's').replace('35s', str(max_weight_len) + 's').format("Layer (type)",
+                                                                                                                                                                 "Output Shape",
+                                                                                                                                                                 "Weight ", "Bias",
+                                                                                                                                                                 "Param #",
+                                                                                                                                                                 "FLOPS #")
+
     print(line_new)
     print("==============================================================================")
-    total_params = 0
+    total_params = np.array([0], dtype=np.float64)
     total_output = 0
-    trainable_params = 0
+    trainable_params = np.array([0], dtype=np.float64)
     flops = np.array([0], dtype=np.float64)
-    macc = 0
+    macc = np.array([0], dtype=np.float64)
     for layer in summary:
         # input_shape, output_shape, trainable, nb_params
         is_keep = '★' if summary[layer]["keep_output"] else ''
@@ -1752,34 +1796,43 @@ def summary(model, input_size, batch_size=1, device="cuda"):
         #     summary[layer]["flops"][0]
         # )
 
-        line_new = "{0:<50s} {1:<25s}  {2:<20s} {3:^8s}  {4:,}  {5:,}  ".format((layer + "  [" + class_name + "]").ljust(50, ' '),
-                                                                                (is_keep + str([None] + summary[layer]["output_shape"][1:])).ljust(25, ' '),
-                                                                                str(summary[layer]["weight"] if 'weight' in summary[layer] else '').ljust(20, ' '),
-                                                                                str(summary[layer]["bias"] if 'bias' in summary[layer] else '').ljust(8, ' '),
-                                                                                summary[layer]["nb_params"],
-                                                                                summary[layer]["flops"].sum()
-                                                                                )
+        line_new = "{0:<50s} {1:<25s}  {2:<35s} {3:<8s}  {4:,.0f}  {5:,.0f}  ".replace('50s', str(max_name_len) + 's').replace('35s', str(max_weight_len) + 's').format(
+            (layer + "  [" + class_name + "]").ljust(max_name_len, ' '),
+            (is_keep + str([None] + summary[layer]["output_shape"][1:])).ljust(25, ' '),
+            str(summary[layer]["weight"].item_list[0] if 'weight' in summary[layer] and len(summary[layer]["weight"]) > 0 else ' ').replace('(', '').replace(')', '').ljust(
+                max_weight_len, ' '),
+            str(summary[layer]["bias"].item_list[0] if 'bias' in summary[layer] and len(summary[layer]["bias"]) > 0 else ' ').replace('(', '').replace(')', '').ljust(8, ' '),
+            summary[layer]["nb_params"],
+            summary[layer]["flops"].sum()
+            )
+        if len(summary[layer]["weight"]) > 1:
+            for n in range(1, len(summary[layer]["weight"])):
+                line_new_add = "{0:<50s} {1:<25s}  {2:<35s} {3:<8s}  {4}  {5}  ".replace('50s', str(max_name_len) + 's').replace('35s', str(max_weight_len) + 's').format(
+                    " ".ljust(max_name_len + len(layer + "  [" + class_name + "]") // 2, " "),
+                    " ".ljust(25 + len(is_keep + str([None] + summary[layer]["output_shape"][1:])) // 2, " "),
+                    str(summary[layer]["weight"].item_list[n] if n < len(summary[layer]["weight"]) else ' ').replace('(', '').replace(')', '').ljust(max_weight_len, " "),
+                    str(summary[layer]["bias"].item_list[n] if n < len(summary[layer]["bias"]) else ' ').replace('(', '').replace(')', '').ljust(8, " "), " ", " ")
+                line_new = line_new + '\n' + line_new_add
 
         total_params += summary[layer]["nb_params"]
         flops += float(summary[layer]["flops"])
         macc += float(summary[layer]["macc"].sum())
         total_output += np.prod(summary[layer]["output_shape"])
         if "trainable" in summary[layer]:
-            if summary[layer]["trainable"] == True:
-                trainable_params += summary[layer]["nb_params"]
+            trainable_params += summary[layer]["trainable"]
         print(line_new)
 
     # assume 4 bytes/number (float on cuda).
-    total_input_size = np.asarray([np.abs(np.prod(to_numpy(shp.dims[1:])) * batch_size * 4. / (1024 ** 2.)) for shp in input_size]).sum()
+    total_input_size = np.asarray([np.abs(np.prod(to_numpy(spec.shape.dims[1:])) * batch_size * 4. / (1024 ** 2.)) for spec in input_specs]).sum()
     total_output_size = np.abs(2. * total_output * 4. / (1024 ** 2.))  # x2 for gradients
     total_params_size = np.abs(total_params * 4. / (1024 ** 2.))
     total_size = total_params_size + total_output_size + total_input_size
 
     print("================================================================")
-    print("Total params: {0:,}".format(total_params))
-    print("Trainable params: {0:,}".format(trainable_params))
-    print("Non-trainable params: {0:,}".format(total_params - trainable_params))
-    print("Total MACC: {0:,}".format(int(macc)))
+    print("Total params: {0:,.0f}".format(total_params[0]))
+    print("Trainable params: {0:,.0f}".format(trainable_params[0]))
+    print("Non-trainable params: {0:,.0f}".format(total_params[0] - trainable_params[0]))
+    print("Total MACC: {0:,.0f}".format(macc[0]))
     print("Total FLOPs: {0:.5f} GFLOPs".format(np.round(flops / 10. ** 9, 5)[0]))
     print("----------------------------------------------------------------")
     print("Input size (MB): %0.2f" % total_input_size)
@@ -1914,7 +1967,7 @@ def try_map_args_and_call(fn, data: OrderedDict, data_feed=None):
                     else:
                         raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
                 # print('arg_map',arg_map.key_list)
-                if ctx.amp_available== True and ctx.is_autocast_enabled == True and get_device() == 'cuda':
+                if ctx.amp_available == True and ctx.is_autocast_enabled == True and get_device() == 'cuda':
                     with torch.cuda.amp.autocast():
                         out = fn(*arg_map.value_list)
                 else:
@@ -1936,7 +1989,7 @@ def try_map_args_and_call(fn, data: OrderedDict, data_feed=None):
 
                         raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
                 # print('arg_map', arg_map.key_list)
-                if ctx.amp_available== True and ctx.is_autocast_enabled == True and get_device() == 'cuda':
+                if ctx.amp_available == True and ctx.is_autocast_enabled == True and get_device() == 'cuda':
                     with torch.cuda.amp.autocast():
                         out = fn(*arg_map.value_list)
                 else:
@@ -1956,7 +2009,7 @@ def try_map_args_and_call(fn, data: OrderedDict, data_feed=None):
                     else:
                         arg_map[arg] = ''
                 # print('arg_map', arg_map.key_list)
-                if ctx.amp_available== True and ctx.is_autocast_enabled == True and get_device() == 'cuda':
+                if ctx.amp_available == True and ctx.is_autocast_enabled == True and get_device() == 'cuda':
                     with torch.cuda.amp.autocast():
                         out = fn(*arg_map.value_list)
                 else:
@@ -1995,6 +2048,10 @@ def fix_layer(layer: Layer):
     Returns: fixed layer
 
     """
+    if not hasattr(layer, 'is_root'):
+        layer.is_root = True
+    if not hasattr(layer, '_device'):
+        layer._device = get_device()
 
     def get_root(self):
         if not hasattr(self, '_nodes') or self._nodes is None or len(self._nodes) < 2:
@@ -2010,8 +2067,7 @@ def fix_layer(layer: Layer):
     layer.to(get_device())
     if not hasattr(layer, '_nodes'):
         layer._nodes = OrderedDict()
-    if not hasattr(layer, 'is_root'):
-        layer.is_root = True
+
     if not hasattr(layer, '_uid_prefixs'):
         layer._uid_prefixs = {}
     reset_name(layer, layer._uid_prefixs)
@@ -2022,15 +2078,37 @@ def fix_layer(layer: Layer):
         if hasattr(layer, 'detection_threshold'):
             delattr(layer, 'detection_threshold')
 
-    if layer._input_shape is not None and isinstance(layer._input_shape, tuple):
+    if layer._input_shape is not None and isinstance(layer._input_shape, tuple) and all([isinstance(d, numbers.Integral) for d in layer._input_shape]):
+        dims = [d for d in layer._input_shape]
+        dims.insert(0, None)
+        layer._input_shape = TensorShape(dims)
+    elif layer._input_shape is not None and isinstance(layer._input_shape, tuple):
         layer._input_shape = tuple([TensorShape(to_numpy(item)) for item in TensorShape(layer._input_shape)])
+    elif layer._input_shape is not None and is_tensor(layer._input_shape):
+        dims = [d.item() for d in layer._input_shape]
+        dims.insert(0, None)
+        buffers = layer.__dict__.get('_buffers')
+        if '_input_shape' in buffers:
+            del buffers['_input_shape']
+        object.__setattr__(layer, '_input_shape', TensorShape(dims))
     elif layer._input_shape is not None and not isinstance(layer._input_shape, TensorShape):
         layer._input_shape = TensorShape(to_numpy(layer._input_shape))
     elif layer._input_shape is not None and isinstance(layer._input_shape, TensorShape) and is_tensor(layer._input_shape.dims[0]):
         layer._input_shape = TensorShape([d.item() for d in layer._input_shape.dims])
 
-    if layer._output_shape is not None and isinstance(layer._output_shape, tuple):
+    if layer._output_shape is not None and isinstance(layer._output_shape, tuple) and all([isinstance(d, numbers.Integral) for d in layer._output_shape]):
+        dims = [d for d in layer._input_shape]
+        dims.insert(0, None)
+        layer._input_shape = TensorShape(dims)
+    elif layer._output_shape is not None and isinstance(layer._output_shape, tuple):
         layer._output_shape = tuple([TensorShape(to_numpy(item)) for item in TensorShape(layer._output_shape)])
+    elif layer._output_shape is not None and is_tensor(layer._output_shape):
+        dims = [d.item() for d in layer._output_shape]
+        dims.insert(0, None)
+        buffers = layer.__dict__.get('_buffers')
+        if '_input_shape' in buffers:
+            del buffers['_output_shape']
+        object.__setattr__(layer, '_output_shape', TensorShape(dims))
     elif layer._output_shape is not None and not isinstance(layer._output_shape, TensorShape):
         layer._output_shape = TensorShape(layer._output_shape)
     elif layer._output_shape is not None and isinstance(layer._output_shape, TensorShape) and isinstance(layer._output_shape.dims[0], torch.Size) and len(
@@ -2039,11 +2117,15 @@ def fix_layer(layer: Layer):
     elif layer._output_shape is not None and isinstance(layer._output_shape, TensorShape) and isinstance(layer._output_shape.dims[0], torch.Size):
         layer._output_shape = tuple([TensorShape(to_numpy(d)) for d in layer._output_shape.dims])
 
-    if not hasattr(layer, 'get_toot'):
+    if not hasattr(layer, 'get_root'):
         setattr(layer, 'get_root', MethodType(get_root, layer))
 
     for module in layer.modules():
         class_name = module.__class__.__name__
+        if not hasattr(layer, 'get_root'):
+            setattr(layer, 'get_root', MethodType(get_root, layer))
+        if not hasattr(module, 'uuid'):
+            module.uuid = uuid.uuid4().node
         # check for root
         if module.uuid == layer.uuid:
             module.is_root = True
@@ -2057,8 +2139,6 @@ def fix_layer(layer: Layer):
         if not hasattr(module, '_default_name') or (module._default_name is None or len(module._default_name) == 0):
             module_prefix = module.__class__.__name__
             module._default_name = camel2snake(module_prefix) + '_' + str(get_global_uid(camel2snake(module_prefix)))
-        if not hasattr(module, 'get_toot'):
-            setattr(module, 'get_root', MethodType(get_root, module))
 
         if not hasattr(module, '_name'):
             module._name = None
@@ -2078,16 +2158,12 @@ def fix_layer(layer: Layer):
         if not hasattr(module, '_non_persistent_buffers_set'):
             module._non_persistent_buffers_set = set()
 
-        if not hasattr(module, 'input_spec'):
-            module.input_spec = None
-            if module.input_shape is not None:
-                module.input_spec = TensorSpec(shape=TensorShape(module._input_shape))
         # fix for shape definition
-        if not isinstance(module._input_shape, TensorShape):
-            module._input_shape = TensorShape(to_numpy(module._input_shape))
-
-        if not isinstance(module._output_shape, TensorShape):
-            module._output_shape = TensorShape(to_numpy(module._output_shape))
+        # if not isinstance(module._input_shape, TensorShape):
+        #     module._input_shape = TensorShape(to_numpy(module._input_shape))
+        #
+        # if not isinstance(module._output_shape, TensorShape):
+        #     module._output_shape = TensorShape(to_numpy(module._output_shape))
 
         if not hasattr(module, 'batch_index'):
             setattr(module, 'batch_index', 0)
@@ -2115,24 +2191,21 @@ def fix_layer(layer: Layer):
             if not hasattr(module, 'use_spectral'):
                 module.use_spectral = False
 
-    if layer.is_root == True and (hasattr(layer, 'signature') or layer._signature is None or (is_tensor(layer._signature.inputs[0].shape))):
+    if layer.is_root == True and (not hasattr(layer, '_signature') or layer._signature is None or len(layer._signature.inputs) == 0):
         layer._signature = Signature()
         if layer._input_shape is not None:
-            if is_tensor(layer._input_shape):
-                layer._signature.inputs["input"] = TensorSpec(shape=TensorShape(layer._input_shape), name="input")
-            elif isinstance(layer._input_shape, tuple) and isinstance(layer._input_shape[0], int):
-                layer._signature.inputs["input"] = TensorSpec(shape=TensorShape(layer._input_shape), name="input")
+            if isinstance(layer._input_shape, TensorSpec):
+                layer._signature.inputs["input"] = TensorSpec(shape=layer._input_shape, name="input")
             elif isinstance(layer._input_shape, tuple):
                 for i in range(len(layer._input_shape)):
-                    layer._signature.inputs["input_{0}".format(i)] = TensorSpec(shape=TensorShape(layer._input_shape[i]), name="input_{0}".format(i))
+                    layer._signature.inputs["input_{0}".format(i)] = TensorSpec(shape=layer._input_shape[i], name="input_{0}".format(i))
         if layer._output_shape is not None:
-            if is_tensor(layer._output_shape):
-                layer._signature.outputs["output"] = TensorSpec(shape=TensorShape(layer._output_shape), name="output")
-            elif isinstance(layer._output_shape, tuple) and isinstance(layer._output_shape[0], int):
-                layer._signature.outputs["output"] = TensorSpec(shape=TensorShape(layer._output_shape), name="output")
+
+            if isinstance(layer._output_shape, TensorSpec):
+                layer._signature.outputs["output"] = TensorSpec(shape=layer._output_shape, name="output")
             elif isinstance(layer._output_shape, tuple):
                 for i in range(len(layer._output_shape)):
-                    layer._signature.outputs["output_{0}".format(i)] = TensorSpec(shape=TensorShape(layer._output_shape[i]), name="output_{0}".format(i))
+                    layer._signature.outputs["output_{0}".format(i)] = TensorSpec(shape=layer._output_shape[i], name="output_{0}".format(i))
         layer.signature = layer._signature
 
     return layer
