@@ -1,12 +1,15 @@
+import copy
 import inspect
 import numbers
+import sys
+from collections import Counter
 from enum import Enum
 from inspect import signature
-from trident.backend.common import to_list, OrderedDict, Signature, split_path, unpack_singleton, get_session,get_backend,TensorShape
+from trident.backend.common import to_list, OrderedDict, Signature, split_path, unpack_singleton, get_session, get_backend, TensorShape, dtype
 from typing import Optional, Union, overload
 import numpy as np
 
-__all__ = ['TensorSpec', 'ObjectType', 'assert_input_compatibility', 'assert_spec_compatibility', 'get_python_function_arguments', 'get_signature', 'ExpectDataType']
+__all__ = ['TensorSpec', 'ObjectType', 'assert_input_compatibility', 'assert_spec_compatibility', 'get_python_function_arguments', 'get_signature', 'ExpectDataType','object_type_inference','distict_color_count']
 
 
 if get_backend()== 'pytorch':
@@ -14,13 +17,13 @@ if get_backend()== 'pytorch':
 elif get_backend() == 'tensorflow':
     from trident.backend.tensorflow_ops  import *
 
-
+_primes=[1, 2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113, 127, 131, 137, 139, 149, 151, 157, 163, 167, 173, 179, 181, 191, 193, 197, 199, 211, 223, 227, 229, 233, 239, 241, 251]
 
 class ObjectType(Enum):
     array_data = 'array_data'
-    gray = 'gray'
-    rgb = 'rgb'
-    rgba = 'rgba'
+    gray = 'gray_image'
+    rgb = 'rgb_image'
+    rgba = 'rgba_image'
     label_mask = 'label_mask'
     color_mask = 'color_mask'
     binary_mask = 'binary_mask'
@@ -33,6 +36,80 @@ class ObjectType(Enum):
     classification_label = 'classification_label'
     corpus = 'corpus'
     sequence_label='sequence_label'
+    sequence_mask = 'sequence_mask'
+    embedding='embedding'
+
+
+
+def distict_color_count(img):
+    """Count for distinct colors in input image
+
+    Returns:
+        object:
+
+    Examples:
+        >>> img=read_image('../../trident_logo.png')[:,:,:3]
+        >>> len(distict_color_count(img))
+        1502
+        >>> img=read_image('../../trident_logo.png')[:,:,:1]
+        >>> len(distict_color_count(img))
+        125
+
+    """
+    return Counter([tuple(colors) for i in img for colors in i])
+
+def pixel_numbers(data):
+    return list(set(data.reshape(-1).tolist()))
+
+def object_type_inference(data):
+    if isinstance(data,np.ndarray):
+        if data.ndim == 2 and data.shape[-1] == 2:
+            return ObjectType.landmarks
+        elif data.ndim == 2 and data.shape[-1] in (4, 5) and 0<=data.max().round(0)<=255 and 0<=data.min().round(0)<=255:
+            return ObjectType.absolute_bbox
+        elif data.ndim == 2 and data.shape[-1] in (4, 5) and 0<=data.max().round(0)<=1 and 0<=data.min().round(0)<=1:
+            return ObjectType.relative_bbox
+        elif data.ndim == 2 and len(distict_color_count(np.expand_dims(data,-1)))==2:
+            return ObjectType.binary_mask
+        elif data.ndim == 2 and (data.max()-data.min()+1)== len(distict_color_count(np.expand_dims(data,-1))):
+            return ObjectType.label_mask
+        elif data.ndim == 2 and 0<=data.max().round(0)<=255 and 0<=data.min().round(0)<=255:
+            return ObjectType.gray
+        elif data.ndim == 3 and (data.shape[-1] == 1 or data.shape[0]==1 ) and len(distict_color_count(data))==2:
+            return ObjectType.binary_mask
+        elif data.ndim == 3 and (data.shape[-1] == 1 or data.shape[0]==1) and (data.max()-data.min()+1)== len(distict_color_count(data)):
+            return ObjectType.label_mask
+        elif data.ndim == 3 and (data.shape[-1] == 1 or data.shape[0]==1)and 0<=data.max().round(0)<=255 and 0<=data.min().round(0)<=255:
+            return ObjectType.gray
+        elif data.ndim == 3 and (data.shape[-1] == 3 or data.shape[0]==3) and 0<=data.max().round(0)<=255 and 0<=data.min().round(0)<=255 :
+            if np.array_equal(data[:,:,0],data[:,:,1]) and np.array_equal(data[:,:,2],data[:,:,1]):
+                return ObjectType.gray
+            elif np.array_equal(data[0,:,:],data[0,:,:]) and np.array_equal(data[2,:,:],data[2,:,:]):
+                return ObjectType.gray
+            elif not np.array_equal(data,data.round()):
+                return ObjectType.rgb
+            elif len(list(set(pixel_numbers(data)) & set(_primes[3:])))>3 :
+                return ObjectType.rgb
+            elif len(distict_color_count(data)) < 100:
+                return ObjectType.color_mask
+            else:
+                return ObjectType.rgb
+        elif data.ndim == 3 and data.shape[-1] == 4 and 0<=data.max().round(0)<=255 and 0<=data.min().round(0)<=255:
+            return ObjectType.rgba
+        elif data.ndim == 3 and data.dtype==np.int64 and 0<=data.max().round(0)<=1 and 0<=data.min().round(0)<=1:
+            return ObjectType.binary_mask
+        elif data.ndim == 3 and data.dtype in [np.float32,np.float16] and 0<=data.max()<=1 and 0<=data.min().round(0)<=1:
+            return ObjectType.alpha_mask
+        elif data.ndim <= 1 and data.dtype==np.int64 :
+            return ObjectType.classification_label
+        elif data.ndim == 2 and data.dtype==np.int64:
+            return ObjectType.color_mask
+        else:
+            sys.stderr.write('Object type cannot be inferred: shape:{0} dtype:{1} min:{2} max:{3} .'.format(data.shape,data.dtype,data.min(),data.max())+'\n')
+            return ObjectType.array_data
+    elif isinstance(data,str):
+        return ObjectType.corpus
+
 
 
 ExpectDataType = ObjectType
@@ -84,11 +161,7 @@ class TensorSpec(object):
         self._dtype = dtype if dtype is not None else None
         self._shape_tuple = None
         self.object_type = object_type
-        if object_type is not None:
-            if 'mask' in object_type.value or 'bbox' in object_type.value or 'rgb' in object_type.value or object_type == ObjectType.gray or object_type == ObjectType.landmarks:
-                self.is_spatial = True
-            else:
-                self.is_spatial = is_spatial
+
         self._name = name
         if shape is not None:
             if isinstance(shape,TensorShape) :
@@ -133,9 +206,23 @@ class TensorSpec(object):
             if max_axis > max_dim:
                 raise ValueError('Axis {} is greater than the maximum allowed value: {}'
                                  .format(max_axis, max_dim))
+
+    def __setattr__(self, name, value):
+        object.__setattr__(self, name, value)
+        if name in ['shape']:
+            self.ndim = self.shape.ndims
+            self._shape_tuple = tuple(self.shape.dims)
+    def copy(self):
+        return copy.deepcopy(self)
+
+
+
     @classmethod
-    def tensor_to_spec(cls, t:Tensor, object_type:ObjectType=None,name=None):
-        return cls(shape=tensor_to_shape(t),dtype=t.dtype,object_type=object_type,name=t.name if name is None else name)
+    def tensor_to_spec(cls, t:Tensor, object_type:ObjectType=None,need_exclude_batch_axis=True, is_singleton=False, name=None):
+        if isinstance(t,numbers.Number):
+            return cls(shape=tensor_to_shape(t, need_exclude_batch_axis=need_exclude_batch_axis, is_singleton=True), dtype=dtype.int64, object_type=object_type, name=name)
+
+        return cls(shape=tensor_to_shape(t,need_exclude_batch_axis=need_exclude_batch_axis,is_singleton=is_singleton),dtype=t.dtype,object_type=object_type,name=name)
 
 
     @property
@@ -148,6 +235,13 @@ class TensorSpec(object):
         """Returns the (optionally provided) name of the described tensor."""
         return self._name
 
+    @property
+    def is_spatial(self):
+         if self.object_type is not None:
+            if 'mask' in self.object_type.value or 'bbox' in self.object_type.value or 'rgb' in self.object_type.value or self.object_type == ObjectType.gray or self.object_type == ObjectType.landmarks:
+                return True
+         return False
+
     def is_compatible_with(self, inputs):  # pylint:disable=useless-super-delegation
         """Returns True if spec_or_tensor is compatible with this TensorSpec.
         Two tensors are considered compatible if they have the same dtype
@@ -465,6 +559,7 @@ def get_signature(fn, name=None):
 
     signature = Signature()
     func_code = fn.__code__
+
     annotations = fn.__annotations__
     sig = inspect.signature(fn)
     paras = list(sig.parameters.items())
@@ -476,7 +571,10 @@ def get_signature(fn, name=None):
 
     for p in paras:
         if p[0] not in ['kwargs', 'self', 'args'] and p[1].default is inspect._empty:
-            signature.inputs[p[0]] = None
+            if p[0]=='x' and fn.__name__=='forward':
+                signature.inputs['input'] = None
+            else:
+                signature.inputs[p[0]] = None
 
     if name is not None:
         signature.name = name
