@@ -300,7 +300,7 @@ class LambdaCallback(CallbackBase):
     Objects of derived classes inject functionality in several points of the training process.
     """
 
-    def __init__(self,when='on_batch_end',epoch=None,batch=None,epoch_frequency=None,batch_frequency=None,action=None,is_shared=False):
+    def __init__(self,when='on_batch_end',frequency=None,unit='batch',action=None,is_shared=False):
         super(LambdaCallback, self).__init__(is_shared=is_shared)
         self.is_shared=is_shared
         self.action = None
@@ -315,44 +315,25 @@ class LambdaCallback(CallbackBase):
             self.when=when
         else:
             raise ValueError("{0} is not valid event trigger.".format(when))
-        self.epoch=epoch
-        self.batch=batch
-        self.epoch_frequency=epoch_frequency
-        self.batch_frequency=batch_frequency
+        self.unit=unit
+        self.frequency=frequency
 
         def on_trigger(self, training_context):
-            if (('epoch' in when and 'batch' not in when)  and
-                ((self.epoch is None and self.epoch_frequency is None)
-                 or training_context['current_epoch']==self.epoch
-                 or (training_context['current_epoch']+1)%self.epoch_frequency==0 ))  :
+            steps=training_context['steps']
+            epoch=training_context['current_epoch']
+            if self.unit=='epoch' and (epoch+1)%self.frequency==0:
                 if ctx.amp_available== True and ctx.is_autocast_enabled  == True and get_device() == 'cuda':
                     with torch.cuda.amp.autocast():
                         self.action(training_context)
                 else:
                     self.action(training_context)
-
-            elif  (('batch' in when and 'epoch' not in when) and
-                   ((self.batch is None and self.batch_frequency is None)
-                or training_context['current_batch']==self.batch
-                or (training_context['steps']+1)%self.batch_frequency==0)):
+            elif self.unit=='batch' and (steps+1)%self.frequency==0:
                 if ctx.amp_available == True and ctx.is_autocast_enabled == True and get_device() == 'cuda':
                     with torch.cuda.amp.autocast():
                         self.action(training_context)
                 else:
                     self.action(training_context)
 
-            else:
-                if ctx.amp_available== True and ctx.is_autocast_enabled == True and get_device() == 'cuda':
-                    with torch.cuda.amp.autocast():
-                        self.action(training_context)
-                else:
-                    self.action(training_context)
-            # elif (('epoch' not  in when and 'batch' not in when)  and ( training_context['current_epoch'] == self.epoch or (
-            #         training_context['current_epoch'] + 1) % self.epoch_frequency == 0)) or (('epoch' not  in when and 'batch' not in when) and (
-            #         training_context['current_batch'] == self.batch or (training_context['steps'] + 1) % self.batch_frequency == 0)):
-            #     self.action(training_context)
-
-
         setattr(self,when,MethodType(on_trigger, self))
 
 
