@@ -2,7 +2,8 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-
+import string
+import codecs
 import pickle
 import io
 import re
@@ -11,15 +12,18 @@ from xml.etree import ElementTree
 import numpy as np
 import copy
 import cv2
+from trident.data.text_transforms import ToHalfWidth, ChineseConvert,bpmf_phonetic
+
 from trident.data.utils import _delete_h
 
 from trident.backend.opencv_backend import image2array, array2image
 from trident.data.image_common import list_images
 from trident.data.data_provider import *
 from trident.data.dataset import *
+from trident.data.text_transforms import *
 from trident.data.mask_common import *
 from trident.data.utils import *
-from trident.backend.common import floatx,OrderedDict
+from trident.backend.common import floatx,OrderedDict,is_alphabet,is_punctuation, remove_nonprintable
 from trident.backend.tensorspec import *
 from trident.misc.ipython_utils import *
 
@@ -292,7 +296,7 @@ def load_birdsnap(dataset_name='birdsnap', kind='train', is_flatten=None, is_one
     return (images, labels)
 
 
-def load_text(filname=None, data=None, label=None,unit='char',mode='next_word',section_delimiter='\n\n',sequence_start_at='random',is_onehot=False,encoding='utf-8-sig',sequence_length=64, return_corpus=False,**kwargs):
+def load_text(filname=None, data=None, label=None,unit='char',mode='next_word',section_delimiter='\n\n',sequence_start_at='section_start',is_onehot=False,encoding='utf-8-sig',sequence_length=64, return_corpus=False,**kwargs):
     valid_sequence_start_at=['random','slide','follow_up','section_start']
     valid_mode=['next_word','skip_gram', 'cbow','onehot','1to1_seq2seq']
     if mode not in valid_mode:
@@ -303,35 +307,42 @@ def load_text(filname=None, data=None, label=None,unit='char',mode='next_word',s
     output_corpus=None
     if filname is not None:
         with io.open(filname, encoding=encoding) as f:
-            original_corpus = f.read().lower()
-            if unit == 'char':
-                corpus = list(original_corpus)
-            elif unit == 'word':
-                corpus = original_corpus.split(' \t')
+            original_corpus = f.read().lower().replace('\u3000' ,' ')
+            corpus=original_corpus.splitlines()
+            new_corpus=[]
+            for item in corpus:
+                if len(item) > 0:
+                    new_corpus.append(item)
+                else:
+                    new_corpus[-1]=new_corpus[-1]+section_delimiter
+            corpus=new_corpus
+
     if data is not None:
         if isinstance(data,str):
-            if unit == 'char':
-                corpus = list(data)
-            elif unit == 'word':
-                corpus = data.split(' \t')
+            corpus = data.splitlines()
         elif hasattr(data,"__iter__"):
-            corpus= '\n\n'.join(data)
-            if unit == 'char':
-                corpus = list(corpus)
-            elif unit == 'word':
-                corpus = corpus.replace('\n\n',' \n \n ').split(' \t')
+            corpus=data
+        new_corpus = []
+        for item in corpus:
+            if len(item) > 0:
+                new_corpus.append(item)
+            else:
+                new_corpus[-1] = new_corpus[-1] + section_delimiter
+        corpus = new_corpus
+
     if label is not None:
-        if isinstance(label,str):
-            if unit == 'char':
-                output_corpus = list(label)
-            elif unit == 'word':
-                output_corpus = label.split(' \t')
-        elif hasattr(label,"__iter__"):
-            output_corpus = '\n\n'.join(label)
-            if unit == 'char':
-                output_corpus = list(output_corpus)
-            elif unit == 'word':
-                output_corpus = output_corpus.replace('\n\n',' \n \n ').split(' \t')
+        if isinstance(label, str):
+            output_corpus = data.splitlines()
+        elif hasattr(label, "__iter__"):
+            output_corpus = data
+        new_corpus = []
+        for item in output_corpus:
+            if len(item) > 0:
+                new_corpus.append(item)
+            else:
+                new_corpus[-1] = new_corpus[-1] +section_delimiter
+        output_corpus = new_corpus
+
     dataprovider=None
     if mode=='next_word':
         corpus1=copy.deepcopy(corpus)
@@ -339,24 +350,24 @@ def load_text(filname=None, data=None, label=None,unit='char',mode='next_word',s
         data_seq=TextSequenceDataset(corpus1,sequence_length=sequence_length,is_onehot=is_onehot,symbol='input',sequence_offset=0,sequence_start_at=sequence_start_at,object_type=ObjectType.corpus)
         labels_seq =TextSequenceDataset(corpus2,sequence_length=sequence_length,is_onehot=is_onehot,symbol='label',sequence_offset=1,sequence_start_at=sequence_start_at,object_type=ObjectType.corpus)
         traindata=Iterator(data=data_seq,label=labels_seq)
-        dataprovider = TextSequenceDataProvider(filname.split('/')[-1].strip().split('.')[0],traindata=traindata)
+        dataprovider = TextSequenceDataProvider(filname.split('/')[-1].strip().split('.')[0],traindata=traindata,sequence_length=sequence_length)
     elif mode=='skip_gram':
         corpus1 = copy.deepcopy(corpus)
         corpus2 = copy.deepcopy(corpus)
         data_seq = TextSequenceDataset(corpus1, sequence_length=sequence_length, is_onehot=is_onehot, symbol='input', sequence_offset=0, sequence_start_at=sequence_start_at)
         labels_seq = TextSequenceDataset(corpus2, sequence_length=sequence_length, is_onehot=is_onehot, symbol='label', sequence_offset=[-1,1], sequence_start_at=sequence_start_at)
         traindata = Iterator(data=data_seq, label=labels_seq)
-        dataprovider = TextSequenceDataProvider(filname.split('/')[-1].strip().split('.')[0], traindata=traindata)
+        dataprovider = TextSequenceDataProvider(filname.split('/')[-1].strip().split('.')[0], traindata=traindata,sequence_length=sequence_length)
     elif mode == '1to1_seq2seq':
         if len(corpus)==len(output_corpus):
             data_seq = TextSequenceDataset(corpus, sequence_length=sequence_length, is_onehot=is_onehot, symbol='input', sequence_offset=0, sequence_start_at=sequence_start_at,object_type=ObjectType.corpus)
             labels_seq = TextSequenceDataset(output_corpus, sequence_length=sequence_length, is_onehot=is_onehot, symbol='label', sequence_offset=0, sequence_start_at=sequence_start_at,object_type=ObjectType.sequence_label)
             traindata = Iterator(data=data_seq, label=labels_seq)
-            dataprovider = TextSequenceDataProvider('1to1_seq2seq', traindata=traindata)
+            dataprovider = TextSequenceDataProvider('1to1_seq2seq', traindata=traindata,sequence_length=sequence_length)
         else:
             raise  ValueError('data ({0}) and label({1}) should have the same length in 1to1_seq2seq mide.'.format(len(corpus),len(output_corpus)))
     if return_corpus:
-        return dataprovider,original_corpus
+        return dataprovider,corpus
     else:
         return dataprovider
 
@@ -610,7 +621,7 @@ def load_examples_data(dataset_name):
     """
     dataset_name = dataset_name.strip().lower()
     if dataset_name.lower() not in ['pokemon', 'hanzi', 'animals', 'nsfw', 'simpsons', 'horse2zebra', 'people',
-                                    'autodrive', 'superresolution', 'anpr', 'beauty','antisproofing','facelandmarks','dogs-vs-cats']:
+                                    'autodrive', 'superresolution', 'anpr', 'beauty','antisproofing','facelandmarks','dogs-vs-cats','chinese']:
         raise ValueError('Not a  valid  dataset_name.')
     dataset_name = 'examples_' + dataset_name
     dirname = os.path.join(_trident_dir, dataset_name)
@@ -949,7 +960,117 @@ def load_examples_data(dataset_name):
         extract_archive(tar_file_path, dirname, archive_format='tar')
         data_provider = load_folder_images(dataset_name, dirname)
         return data_provider
+    elif dataset_name == 'examples_chinese':
+        to_half=ToHalfWidth()
+        to_sc=ChineseConvert(convert_to='simplified')
+        download_file_from_google_drive('1yzRzXpLuhSUxnixqCgpbdTk16ajnTEWF', dirname, 'chinese.tar')
+        tar_file_path = os.path.join(dirname, 'chinese.tar')
+        extract_archive(tar_file_path, dirname, archive_format='tar')
+        as_train = remove_nonprintable(to_half(codecs.open(os.path.join(dirname, 'as_training.utf8'), encoding='utf-8-sig').read().strip().replace('  ','|').replace('\u3000' ,'|'))).splitlines()
+
+        cityu_train =remove_nonprintable(to_half(codecs.open(os.path.join(dirname, 'cityu_training.utf8'), encoding='utf-8-sig').read().strip().replace('  ','|').replace('\u3000' ,'|'))).splitlines()
+        #as_test = codecs.open(os.path.join(dirname, 'as_test.utf8'), encoding='utf-8-sig').read()
+
+        #cityu_test = codecs.open(os.path.join(dirname, 'cityu_testing.utf8'), encoding='utf-8-sig').read()
 
 
+        #as_test = as_test.replace('\u3000', '|').replace(' ', '|')  # 把分詞分隔號置換為'|'，否則會被視為空白被處理掉
+        #cityu_test = cityu_test.replace(' ', '|')  # 把分詞分隔號置換為'|'，否則會被視為空白被處理掉
+
+        data = as_train  + cityu_train  # 把兩個語料合併
+        #test_data=as_test + '\r\n' + cityu_test  # 把兩個語料合併
+
+
+
+        raw_data_train = [row.strip('\n').strip('\r').replace("\x08", '').replace("\x80", '') for row in data]  # 移除分行字元
+
+        process_data_train=[]
+        process_seg_label_train = []
+        process_simplifided_label_train = []
+        process_traditional_label_train = []
+        print('generate labels')
+        tmp_data_train = []
+        tmp_seg_label_train = []
+        tmp_simplifided_label_train = []
+        tmp_pronunce_label_train = []
+        for row in tqdm(raw_data_train):
+            words=row.replace('||','|').split('|')
+            for k  in range(len(words)):
+                word=words[k]
+
+                for i in range(len(word)):
+                    tmp_data_train.append(word[i])
+                    #tmp_simplifided_label_train.append(to_half(to_sc(word[i])))
+                    #轉換為BMES
+                    if len(word)==1 and i==0: #S 自己就是一個單詞
+                        tmp_seg_label_train.append('S')
+                    elif i==0: #B 是一個詞的開始
+                        tmp_seg_label_train.append('B')
+                    elif i==len(word)-1:  #E 是一個詞的結束
+                        tmp_seg_label_train.append('E')
+                    else: #M 是一個詞的中間
+                        tmp_seg_label_train.append('M')
+
+                if k<len(words) and len(word)>0 and (is_alphabet(word) or is_punctuation(word)):
+                    if k==len(words)-1 and word=="。":
+                        pass
+                    else:
+                        tmp_data_train.append(' ')
+                        tmp_seg_label_train.append('S')
+
+            if words[-1]=="。":
+                #process_traditional_label_train.append(tmp_data_train)
+
+                tmp_data_train=''.join(tmp_data_train)
+                tmp_seg_label_train = ''.join(tmp_seg_label_train)
+                tmp_simplifided_label_train =to_half(to_sc(tmp_data_train))
+
+                process_data_train.append(tmp_data_train)
+                process_seg_label_train.append(tmp_seg_label_train)
+                process_simplifided_label_train.append(tmp_simplifided_label_train)
+                tmp_data_train = []
+                tmp_seg_label_train = []
+                tmp_simplifided_label_train = []
+                tmp_pronunce_label_train = []
+            # else:
+            #     tmp_data_train.append('\n')
+            #     tmp_simplifided_label_train.append('\n')
+            #     tmp_seg_label_train.append('\n')
+        corpus=process_data_train
+        seg_corpus=process_seg_label_train
+        simplifided_corpus =process_simplifided_label_train
+        data=TextSequenceDataset(corpus=corpus,sequence_length=128,sequence_start_at='section_start',object_type=ObjectType.corpus,symbol='input')
+        seg_label = TextSequenceDataset(corpus=seg_corpus,sequence_length=128, sequence_start_at='section_start', object_type=ObjectType.sequence_label,symbol='seg_label')
+        simplifided_label = TextSequenceDataset(corpus=simplifided_corpus,sequence_length=128, sequence_start_at='section_start', object_type=ObjectType.sequence_label,symbol='simplified_label')
+        traditional_label = TextSequenceDataset(corpus= copy.deepcopy(corpus), sequence_length=128, sequence_start_at='section_start', object_type=ObjectType.sequence_label,symbol='traditional_label')
+
+        chars = list(sorted(set(list( ''.join(corpus) +bpmf_phonetic+'\n\r\t'+  ''.join(simplifided_corpus)))))
+        chars.insert(0, '[CLS]')
+        chars.insert(1, '[SEP]')
+        chars.insert(2, '[UNK]')
+        chars.insert(3, '[PAD]')
+        chars.insert(4, '[MASK]')
+
+        data.vocabs =simplifided_label.vocabs =  chars
+        data.text2index =simplifided_label.text2index =  dict((c, i) for i, c in enumerate(chars))
+        data.index2text =simplifided_label.index2text  =  dict((i, c) for i, c in enumerate(chars))
+        traditional_label = copy.deepcopy(data)
+        traditional_label.object_type = ObjectType.sequence_label
+        traditional_label.symbol = 'traditional_label'
+
+        mask_label = copy.deepcopy(data)
+        mask_label.object_type = ObjectType.corpus
+        mask_label.symbol = 'mask_label'
+
+
+
+        nextword=copy.deepcopy(data)
+        nextword.object_type=ObjectType.sequence_label
+        nextword.symbol='nextword_label'
+        nextword.sequence_offset=1
+
+        label=ZipDataset(seg_label,nextword,simplifided_label,traditional_label,mask_label)
+        provider=TextSequenceDataProvider(traindata=Iterator(data=data,label=label,sample_filter=lambda x:x[0][30]!=3))
+        return provider
     else:
         return None
\ No newline at end of file
