@@ -207,17 +207,16 @@ class ZipDataset(Dataset):
 
     def __init__(self, *datasets, **kwargs):
         """See `Dataset.zip()` for details."""
-        super().__init__()
+        super().__init__(datasets, **kwargs)
         if not all(len(datasets[0]) == len(ds) for ds in datasets):
             raise ValueError("lengths of input datasets are inconsistent")
-        self._datasets = datasets
         self.symbol = tuple([ds.symbol for ds in datasets])
 
     def __getitem__(self, index: int) -> Tuple:
-        return tuple(ds[index] for ds in self._datasets)
+        return tuple(ds[index] for ds in self.items)
 
     def __len__(self) -> int:
-        return len(self._datasets[0])
+        return len(self.items[0])
 
 
 class NumpyDataset(Dataset):
@@ -320,7 +319,6 @@ class ImageDataset(Dataset):
                 if (inspect.isfunction(fc) or isinstance(fc, Transform)) and fc is not image_backend_adaption:
                     img_data = fc(img_data, spec=self.element_spec)
             img_data = image_backend_adaption(img_data)
-
             return img_data
         else:
             return img_data
@@ -625,85 +623,248 @@ class RandomNoiseDataset(Dataset):
         return self.length
 
 
+# class TextSequenceDataset(Dataset):
+#     def __init__(self, corpus=None, is_onehot=False, sequence_offset=0, section_delimiter='\n\n', stopwords=None, sequence_length: int = 64, sequence_start_at='random',
+#                  object_type=ObjectType.corpus, symbol=None, **kwargs):
+#         super().__init__(None,symbol=symbol, object_type=object_type, **kwargs)
+#         self.sequence_start_at = sequence_start_at
+#         self.transform_funcs = []
+#         if len(section_delimiter) == 2:
+#             self.section_delimiter = section_delimiter
+#         else:
+#             self.section_delimiter = '\n\n'
+#         self.vocabs = None
+#         self.text2index = None
+#         self.index2text = None
+#         self.is_onehot = is_onehot
+#         self.is_paired_process = False
+#         self.is_spatial = True
+#
+#         if hasattr(corpus, "__iter__"):
+#             new_corpus = []
+#             section = ['<start/>']
+#             for i in range(len(corpus)):
+#                 item = corpus[i]
+#                 if item == '\n' and corpus[i - 1] != '\n' and len(section) > 0:
+#                     section.append('<end/>')
+#                     if i < len(corpus) - 1 and corpus[i + 1] == '\n':
+#                         new_corpus.append(section)
+#                         section = ['<start/>']
+#                     elif i < len(corpus) - 1:
+#                         section.append('<start/>')
+#                 elif self.section_delimiter != '\n\n' and len(self.section_delimiter) == 2 and len(section) > 0 and item == self.section_delimiter[0] and i < len(corpus) - 1 and \
+#                         corpus[i - 1] == self.section_delimiter[1]:
+#                     section.append('<end/>')
+#                     if i < len(corpus) - 1 and corpus[i + 1] == '\n':
+#                         new_corpus.append(section)
+#                         section = ['<start/>']
+#                     elif i < len(corpus) - 1:
+#                         section.append('<start/>')
+#                 elif item == '\r' and corpus[i - 1] == '\n':
+#                     pass
+#                 elif item == '\n':
+#                     pass
+#                 else:
+#                     section.append(item)
+#                     if i == len(corpus) - 1:
+#                         section.append('<end/>')
+#             if len(section) > 0:
+#                 new_corpus.append(section)
+#
+#             if self.sequence_start_at == 'random':
+#                 for sect in new_corpus:
+#                     self.items.extend(sect)
+#                     self.items.append('\n')
+#                     self.items.append('\n')
+#             else:
+#                 self.items.extend(new_corpus)
+#
+#             chars = sorted(list(set(corpus)))
+#
+#             chars.insert(0, '<start/>')
+#             chars.insert(1, '<end/>')
+#             chars.insert(2, '[UNK]')
+#             chars.insert(3, '[PAD]')
+#             print('total distinct chars:', len(chars))
+#             self.vocabs = chars
+#             self.text2index = dict((c, i) for i, c in enumerate(chars))
+#             self.index2text = dict((i, c) for i, c in enumerate(chars))
+#         else:
+#             raise ValueError('corpus should be a collection.')
+#
+#
+#         self.sequence_offset = sequence_offset
+#         self.dtype = np.float32 if self.is_onehot else np.int64
+#         self.transform_funcs = []
+#         self.is_paired_process = False
+#         self.sequence_length = sequence_length
+#
+#     def _get_item_by_idx(self, iterator, idx):
+#         """Get the idx-th item of the iterator"""
+#         size = len(self)
+#         idx = idx.__index__()
+#         if not -size <= idx < size:
+#             raise IndexError('index {} is out of range'.format(idx))
+#         idx %= size
+#         return next(itertools.islice(iterator, idx, None))
+#
+#     def __getitem__(self, index: int):
+#         sequencetext = None
+#         seq_base = list(self.items)
+#         if isinstance(self.sequence_offset, int):
+#             if self.sequence_start_at == 'random':
+#                 sequencetext = seq_base[index + self.sequence_offset:builtins.min(index + self.sequence_offset + self.sequence_length, self.len())]
+#             elif self.sequence_start_at == 'section_start':
+#                 sectiontext = seq_base[index]
+#                 sequencetext = sectiontext[self.sequence_offset:builtins.min(self.sequence_offset + self.sequence_length, self.len())]
+#         elif isinstance(self.sequence_offset, list):
+#             sequencetext = []
+#             if self.sequence_start_at == 'random':
+#                 for k in self.sequence_offset:
+#                     if 0 <= index + self.sequence_offset < len(seq_base):
+#                         sequencetext.append(seq_base[index + self.sequence_offset])
+#                     else:
+#                         sequencetext.append('[PAD]')
+#
+#             elif self.sequence_start_at == 'section_start':
+#                 sectiontext = seq_base[index]
+#                 for k in self.sequence_offset:
+#                     if 0 <= index + self.sequence_offset < len(seq_base):
+#                         sequencetext.append(seq_base[index + self.sequence_offset])
+#                     else:
+#                         sequencetext.append('[PAD]')
+#
+#         if len(sequencetext) != self.sequence_length:
+#             sequencetext.extend(['[PAD]'] * (self.sequence_length - len(sequencetext)))
+#         arr = None
+#         if self.is_onehot:
+#             arr = np.zeros((self.sequence_length, len(self.text2index)))
+#             for i in range(self.sequence_length):
+#                 this_char = sequencetext[i]
+#                 if i + 1 < self.sequence_length and ''.join(sequencetext[i:i + 2]) == '\n\n':
+#                     arr[i:, self.text2index['[PAD]']] = 1
+#                     break
+#                 elif this_char in self.text2index:
+#                     arr[i, self.text2index[this_char]] = 1
+#                 else:
+#                     arr[i, self.text2index['[UNK]']] = 1
+#             arr = arr.astype(np.float32)
+#         else:
+#             arr = np.zeros((self.sequence_length))
+#             for i in range(self.sequence_length):
+#                 this_char = sequencetext[i]
+#                 if i + 1 < self.sequence_length and ''.join(sequencetext[i:i + 2]) == '\n\n':
+#                     arr[i:] = self.text2index['[PAD]']
+#                     break
+#                 elif this_char in self.text2index:
+#                     arr[i] = self.text2index[this_char]
+#                 else:
+#                     arr[i] = self.text2index['[UNK]']
+#             arr = arr.astype(np.int64)
+#
+#         if self.is_paired_process == False and len(self.transform_funcs) == 0:
+#             return text_backend_adaption(arr)
+#         elif not self.is_paired_process:
+#             return self.text_transform(arr)
+#         elif self.is_paired_process:
+#             return arr
+#
+#         return None
+#
+#     def data_transform(self, text_data):
+#         if len(self.transform_funcs) == 0:
+#             return text_backend_adaption(text_data)
+#         if isinstance(text_data, np.ndarray):
+#             for fc in self.transform_funcs:
+#                 text_data = fc(text_data, spec=self.element_spec)
+#             text_data = text_backend_adaption(text_data)
+#
+#     def text_transform(self, text_data):
+#         return self.data_transform(text_data)
+#
+#     @property
+#     def reverse_transform_funcs(self):
+#         return_list = []
+#         return_list.append(reverse_text_backend_adaption)
+#         for i in range(len(self.transform_funcs)):
+#             fn = self.transform_funcs[-1 - i]
+#             # if fn.__qualname__ == 'normalize.<locals>.text_op':
+#             #     return_list.append(unnormalize(fn.mean, fn.std))
+#         # return_list.append(array2image)
+#         return return_list
+#
+#     def reverse_text_transform(self, text_data):
+#         if len(self.reverse_transform_funcs) == 0:
+#             return reverse_text_backend_adaption(text_data)
+#         if isinstance(text_data, np.ndarray):
+#             # if img_data.ndim>=2:
+#             for fc in self.reverse_transform_funcs:
+#                 text_data = fc(text_data)
+#             text_data = reverse_text_backend_adaption(text_data)
+#
+#             return text_data
+#         else:
+#             return text_data
+#
+
 class TextSequenceDataset(Dataset):
-    def __init__(self, corpus=None, is_onehot=False, sequence_offset=0, section_delimiter='\n\n', stopwords=None, sequence_length: int = 64, sequence_start_at='random',
+    def __init__(self, corpus=None, is_onehot=False, sequence_offset=0, storage_unit='section',section_delimiter='\n\n', stopwords=None, sequence_length: int = 64, sequence_start_at='random',
                  object_type=ObjectType.corpus, symbol=None, **kwargs):
         super().__init__(None,symbol=symbol, object_type=object_type, **kwargs)
+        self.sequence_length = sequence_length
         self.sequence_start_at = sequence_start_at
         self.transform_funcs = []
         if len(section_delimiter) == 2:
             self.section_delimiter = section_delimiter
         else:
             self.section_delimiter = '\n\n'
+        self.storage_unit=storage_unit
         self.vocabs = None
         self.text2index = None
         self.index2text = None
         self.is_onehot = is_onehot
         self.is_paired_process = False
         self.is_spatial = True
+        self.add_corpus(corpus)
+        self.sequence_offset = sequence_offset
+        self.dtype = np.float32 if self.is_onehot else np.int64
 
+        self.is_paired_process = False
+
+
+    def add_corpus(self,corpus):
         if hasattr(corpus, "__iter__"):
-            new_corpus = []
-            section = []
-            section.append('<start/>')
-            for i in range(len(corpus)):
-                item = corpus[i]
-                if item == '\n' and corpus[i - 1] != '\n' and len(section) > 0:
-                    section.append('<end/>')
-                    if i < len(corpus) - 1 and corpus[i + 1] == '\n':
-                        new_corpus.append(section)
-                        section = []
-                        section.append('<start/>')
-                    elif i < len(corpus) - 1:
-                        section.append('<start/>')
-                elif self.section_delimiter != '\n\n' and len(self.section_delimiter) == 2 and len(section) > 0 and item == self.section_delimiter[0] and i < len(corpus) - 1 and \
-                        corpus[i - 1] == self.section_delimiter[1]:
-                    section.append('<end/>')
-                    if i < len(corpus) - 1 and corpus[i + 1] == '\n':
-                        new_corpus.append(section)
-                        section = []
-                        section.append('<start/>')
-                    elif i < len(corpus) - 1:
-                        section.append('<start/>')
-                elif item == '\r' and corpus[i - 1] == '\n':
-                    pass
-                elif item == '\n':
-                    pass
-                else:
-                    section.append(item)
-                    if i == len(corpus) - 1:
-                        section.append('<end/>')
-            if len(section) > 0:
-                new_corpus.append(section)
-
-            if self.sequence_start_at == 'random':
-                for sect in new_corpus:
-                    self.items.extend(sect)
-                    self.items.append('\n')
-                    self.items.append('\n')
+            self.items.extend(corpus)
+            if self.vocabs is None:
+                chars = sorted(list(set( ''.join(corpus ))))
+                chars.insert(0, '[CLS]')
+                chars.insert(1, '[SEP]')
+                chars.insert(2, '[UNK]')
+                chars.insert(3, '[PAD]')
+                chars.insert(4, '[MASK]')
+                print('total distinct chars:', len(chars))
+                self.vocabs = chars
+                self.text2index = dict((c, i) for i, c in enumerate(chars))
+                self.index2text = dict((i, c) for i, c in enumerate(chars))
             else:
-                self.items.extend(new_corpus)
+                new_chars = sorted(list(set(''.join(corpus))))
+                new_chars=[ch for ch in new_chars if ch not in self.vocabs]
+                new_chars=list(sorted(set(''.join(new_chars))))
+                for ch in new_chars:
+                    self.vocabs.append(ch)
+                    self.text2index[ch]=len(self.vocabs)-1
+                    self.index2text[len(self.vocabs)-1]=ch
+            self.length_index=OrderedDict()
+            total_len=0
+            for i in range(len(self.items)):
+                total_len+=(len(self.items[i])+2)
+                self.length_index[i]=total_len
 
-            chars = sorted(list(set(corpus)))
 
-            chars.insert(0, '<start/>')
-            chars.insert(1, '<end/>')
-            chars.insert(2, '<unknown/>')
-            chars.insert(3, '<pad/>')
-            print('total distinct chars:', len(chars))
-            self.vocabs = chars
-            self.text2index = dict((c, i) for i, c in enumerate(chars))
-            self.index2text = dict((i, c) for i, c in enumerate(chars))
         else:
             raise ValueError('corpus should be a collection.')
 
 
-        self.sequence_offset = sequence_offset
-        self.dtype = np.float32 if self.is_onehot else np.int64
-        self.text_transform_funcs = []
-        self.is_paired_process = False
-        self.sequence_length = sequence_length
-
     def _get_item_by_idx(self, iterator, idx):
         """Get the idx-th item of the iterator"""
         size = len(self)
@@ -713,97 +874,119 @@ class TextSequenceDataset(Dataset):
         idx %= size
         return next(itertools.islice(iterator, idx, None))
 
+    def __len__(self):
+        if self.sequence_start_at == 'random':
+            return self.length_index.value_list[-1]
+        elif self.sequence_start_at == 'section_start':
+            return len(self.items)
+
     def __getitem__(self, index: int):
         sequencetext = None
-        seq_base = list(self.items)
         if isinstance(self.sequence_offset, int):
+
             if self.sequence_start_at == 'random':
-                sequencetext = seq_base[index + self.sequence_offset:builtins.min(index + self.sequence_offset + self.sequence_length, self.len())]
+                for k,v in self.length_index.item_list:
+                    if v>index:
+                        last_v = self.length_index[k - 1] if k>0 else 0
+                        sectiontext = list(self.items[k])
+                        sectiontext.insert(0, '[CLS]')
+                        is_end=self.items[k].endswith('\n\n')
+                        if is_end:
+                            sectiontext=sectiontext[:-2]
+                        sectiontext.append('[SEP]')
+                        if k+1<len( self.length_index) and not is_end :
+                            sectiontext.append('[CLS]')
+                            sectiontext.extend(list(self.items[k+1]))
+                            is_end = self.items[k+1].endswith('\n\n')
+                            if is_end:
+                                sectiontext = sectiontext[:-2]
+                            sectiontext.append('[SEP]')
+                        if k + 2 < len(self.length_index) and not is_end:
+                            sectiontext.append('[CLS]')
+                            sectiontext.extend(list(self.items[k + 2]))
+                            is_end = self.items[k + 2].endswith('\n\n')
+                            if is_end:
+                                sectiontext = sectiontext[:-2]
+                            sectiontext.append('[SEP]')
+                        idx=index-last_v
+
+                        sequencetext =sectiontext[idx+self.sequence_offset:builtins.min(idx + self.sequence_offset + self.sequence_length,len(sectiontext))]
+                        break
             elif self.sequence_start_at == 'section_start':
-                sectiontext = seq_base[index]
-                sequencetext = sectiontext[self.sequence_offset:builtins.min(self.sequence_offset + self.sequence_length, self.len())]
-        elif isinstance(self.sequence_offset, list):
-            sequencetext = []
-            if self.sequence_start_at == 'random':
-                for k in self.sequence_offset:
-                    if 0 <= index + self.sequence_offset < len(seq_base):
-                        sequencetext.append(seq_base[index + self.sequence_offset])
-                    else:
-                        sequencetext.append('<pad/>')
+                sectiontext = list(self.items[index])
+                sectiontext.insert(0, '[CLS]')
+                sectiontext.append('[SEP]')
+                sequencetext = sectiontext[self.sequence_offset:builtins.min(self.sequence_offset + self.sequence_length,len(sectiontext))]
+        return sequencetext
 
-            elif self.sequence_start_at == 'section_start':
-                sectiontext = seq_base[index]
-                for k in self.sequence_offset:
-                    if 0 <= index + self.sequence_offset < len(seq_base):
-                        sequencetext.append(seq_base[index + self.sequence_offset])
-                    else:
-                        sequencetext.append('<pad/>')
-
-        if len(sequencetext) != self.sequence_length:
-            sequencetext.extend(['<pad/>'] * (self.sequence_length - len(sequencetext)))
-        arr = None
-        if self.is_onehot:
-            arr = np.zeros((self.sequence_length, len(self.text2index)))
-            for i in range(self.sequence_length):
-                this_char = sequencetext[i]
-                if i + 1 < self.sequence_length and ''.join(sequencetext[i:i + 2]) == '\n\n':
-                    arr[i:, self.text2index['<pad/>']] = 1
-                    break
-                elif this_char in self.text2index:
-                    arr[i, self.text2index[this_char]] = 1
-                else:
-                    arr[i, self.text2index['<unknown/>']] = 1
-            arr = arr.astype(np.float32)
-        else:
-            arr = np.zeros((self.sequence_length))
-            for i in range(self.sequence_length):
-                this_char = sequencetext[i]
-                if i + 1 < self.sequence_length and ''.join(sequencetext[i:i + 2]) == '\n\n':
-                    arr[i:] = self.text2index['<pad/>']
-                    break
-                elif this_char in self.text2index:
-                    arr[i] = self.text2index[this_char]
-                else:
-                    arr[i] = self.text2index['<unknown/>']
-            arr = arr.astype(np.int64)
-
-        if self.is_paired_process == False and len(self.text_transform_funcs) == 0:
+        if self.is_paired_process == False and len(self.transform_funcs) == 0:
             return text_backend_adaption(arr)
-        elif self.is_paired_process == False:
+        elif not self.is_paired_process:
             return self.text_transform(arr)
-        elif self.is_paired_process == True:
+        elif self.is_paired_process:
             return arr
 
         return None
 
     def data_transform(self, text_data):
         if len(self.transform_funcs) == 0:
-            return text_backend_adaption(text_data)
-        if isinstance(text_data, np.ndarray):
-            for fc in self.transform_funcs:
-                text_data = fc(text_data, spec=self.element_spec)
-            text_data = text_backend_adaption(text_data)
+            pass
+
+        for fc in self.transform_funcs:
+            text_data = fc(text_data, spec=self.element_spec)
+        #text_data = text_backend_adaption(text_data)
+
+        arr = None
+        if self.is_onehot:
+            arr = np.zeros((self.sequence_length, len(self.text2index)))
+            for i in range(self.sequence_length):
+                if i < len(text_data):
+                    this_char = text_data[i]
+                    if this_char in self.text2index:
+                        arr[i, self.text2index[this_char]] = 1
+                    else:
+                        arr[i, self.text2index['[UNK]']] = 1
+                else:
+                    arr[i:, self.text2index['[PAD]']] = 1
+            arr = arr.astype(np.float32)
+        else:
+            arr = np.zeros((self.sequence_length))
+            for i in range(self.sequence_length):
+                if i<len(text_data):
+                    this_char = text_data[i]
+                    if this_char in self.text2index:
+                        arr[i] = self.text2index[this_char]
+                    else:
+                        arr[i] = self.text2index['[UNK]']
+                else:
+                    arr[i] = self.text2index['[PAD]']
+            arr = arr.astype(np.int64)
+        return arr
+
+
+
+
 
     def text_transform(self, text_data):
         return self.data_transform(text_data)
 
     @property
-    def reverse_text_transform_funcs(self):
+    def reverse_transform_funcs(self):
         return_list = []
         return_list.append(reverse_text_backend_adaption)
-        for i in range(len(self.text_transform_funcs)):
-            fn = self.text_transform_funcs[-1 - i]
+        for i in range(len(self.transform_funcs)):
+            fn = self.transform_funcs[-1 - i]
             # if fn.__qualname__ == 'normalize.<locals>.text_op':
             #     return_list.append(unnormalize(fn.mean, fn.std))
         # return_list.append(array2image)
         return return_list
 
     def reverse_text_transform(self, text_data):
-        if len(self.reverse_text_transform_funcs) == 0:
+        if len(self.reverse_transform_funcs) == 0:
             return reverse_text_backend_adaption(text_data)
         if isinstance(text_data, np.ndarray):
             # if img_data.ndim>=2:
-            for fc in self.reverse_text_transform_funcs:
+            for fc in self.reverse_transform_funcs:
                 text_data = fc(text_data)
             text_data = reverse_text_backend_adaption(text_data)
 
@@ -813,15 +996,18 @@ class TextSequenceDataset(Dataset):
 
 
 class Iterator(object):
-    def __init__(self, data=None, label=None, mask=None, unpair=None, sample_filter=None, minibatch_size=8, mode='tuple', is_shuffe=True, buffer_size=None, workers=2, **kwargs):
+    def __init__(self, data=None, label=None, mask=None, unpair=None, sample_filter=None, batch_size=8, mode='tuple', is_shuffe=True, buffer_size=None, workers=2, **kwargs):
         self.is_paired_process = False
         self._data = None
         self._label = None
         self._unpair = None
         self.paired_process_symbols = []
-        self.data_template = None
+
         self.is_shuffe = is_shuffe
-        self.datasets_dict = OrderedDict()
+
+
+
+        self.data_template = OrderedDict()
         self.mode = mode
 
         self.workers = workers
@@ -848,8 +1034,7 @@ class Iterator(object):
             self._unpair = NumpyDataset(symbol="")
 
         datasets = self.get_datasets()
-        for ds in datasets:
-            self.datasets_dict[ds.symbol] = ds
+
         data_symbols = iteration_tools.flatten([self.data.symbol], iterable_types=(list, tuple))
         label_symbols = iteration_tools.flatten([self.label.symbol], iterable_types=(list, tuple))
         # check pair_process
@@ -864,23 +1049,23 @@ class Iterator(object):
                 ds.is_paired_process = True
                 self.paired_process_symbols.append(ds.symbol)
 
-        self.data_template = OrderedDict()
-
-
         for k in range(len(datasets)):
             ds = datasets[k]
-            if len(ds) > 0:
-                dataitem = ds[k]
-                ds.element_spec = TensorSpec.tensor_to_spec(expand_dims(dataitem, 0), object_type=ds.object_type, name=ds.symbol)
-                self.data_template[ds.element_spec] = None
+            if isinstance(ds,TextSequenceDataset):
+                ds.element_spec = TensorSpec(shape=TensorShape([None,ds.sequence_length]), dtype=dtype.long,object_type=ds.object_type, name=ds.symbol)
+            else:
+                if len(ds) > 0:
+                    dataitem = ds[k]
+                    ds.element_spec = TensorSpec.tensor_to_spec(expand_dims(dataitem, 0), object_type=ds.object_type, name=ds.symbol)
+                    self.data_template[ds.element_spec] = None
 
-        self._signature=None
-        self._minibatch_size = minibatch_size
+        self._batch_size = batch_size
         self.paired_transform_funcs = []
-        self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=self.is_shuffe, drop_last=False)
+
+        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffe, drop_last=False)
         self._sample_iter = iter(self.batch_sampler)
         if buffer_size is None:
-            buffer_size = 8 * minibatch_size
+            buffer_size = 8 * batch_size
         self.buffer_size = buffer_size
         self.out_queue = Queue.Queue(maxsize=self.buffer_size)
         self.sample_filter = None
@@ -903,7 +1088,7 @@ class Iterator(object):
         else:
             self._label.is_paired_process = self._data.is_paired_process = self.is_paired_process = False
 
-        self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=True, drop_last=False)
+        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffe, drop_last=False)
         self.batch_sampler.sample_filter = self.sample_filter
         self._sample_iter = iter(self.batch_sampler)
 
@@ -920,7 +1105,7 @@ class Iterator(object):
             self._label.is_paired_process = self._data.is_paired_process = self.is_paired_process = True
         else:
             self._label.is_paired_process = self._data.is_paired_process = self.is_paired_process = False
-        self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=True, drop_last=False)
+        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffe, drop_last=False)
         self.batch_sampler.sample_filter = self.sample_filter
         self._sample_iter = iter(self.batch_sampler)
 
@@ -933,7 +1118,7 @@ class Iterator(object):
         self._unpair = value
 
 
-        self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=True, drop_last=False)
+        self.batch_sampler = BatchSampler(self, self._batch_size,is_shuffle=self.is_shuffe,  drop_last=False)
         self.batch_sampler.sample_filter = self.sample_filter
         self._sample_iter = iter(self.batch_sampler)
 
@@ -945,13 +1130,13 @@ class Iterator(object):
             return None
 
     @property
-    def minibatch_size(self):
-        return self._minibatch_size
+    def batch_size(self):
+        return self._batch_size
 
-    @minibatch_size.setter
-    def minibatch_size(self, value):
-        self._minibatch_size = value
-        self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=True, drop_last=False)
+    @batch_size.setter
+    def batch_size(self, value):
+        self._batch_size = value
+        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffe, drop_last=False)
         self.batch_sampler.sample_filter = self.sample_filter
         self._sample_iter = iter(self.batch_sampler)
         self.buffer_size = 8 * value
@@ -962,63 +1147,21 @@ class Iterator(object):
     @property
     def signature(self):
         datasets = self.get_datasets()
-        for ds in datasets:
-            if self._signature is not None and ds.symbol not in self._signature.outputs:
-                self._signature =None
-        if self._signature is None:
+        if self._signature is not None and len(self._signature.outputs)==len(datasets):
+            return self._signature
+        else:
             self._signature = Signature(name='data_provider')
-            self.data_template=OrderedDict()
-            self.datasets_dict=OrderedDict()
-            for k in range(len(datasets)):
-                ds = datasets[k]
-                if isinstance(ds, ZipDataset):
-                    for dsitem in ds._datasets:
-                        if len(dsitem) > 0:
-                            dataitem = dsitem[k]
-                            dsitem.element_spec = TensorSpec.tensor_to_spec(expand_dims(dataitem, 0), object_type=ds.object_type, name=ds.symbol)
-                            self.datasets_dict[dsitem.symbol] = dsitem
-                            self.data_template[dsitem.element_spec] = None
-                            self._signature.outputs[dsitem.symbol] = ds.element_spec
-                else:
 
-                    if len(ds) > 0:
-                        dataitem = ds[k]
-                        ds.element_spec = TensorSpec.tensor_to_spec(expand_dims(dataitem, 0), object_type=ds.object_type, name=ds.symbol)
-                        self.datasets_dict[ds.symbol] = ds
-                        self.data_template[ds.element_spec] = None
-                        self._signature.outputs[ds.symbol] = ds.element_spec
-
-            self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=self.is_shuffe, drop_last=False)
+            for ds in datasets:
+                spec = ds.element_spec
+                self.data_template[spec] = None
+                self._signature.outputs[ds.symbol] = spec
+            self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffe, drop_last=False)
             self._sample_iter = iter(self.batch_sampler)
-        return self._signature
+            return self._signature
 
-    # def update_signature(self, arg_names):
-    #
-    #     if len(arg_names) != len(self.datasets_dict):
-    #         warnings.warn('arg_names should have same length with datasets, expect {0} got {1}'.format(len(arg_names), len(self.datasets_dict)))
-    #     else:
-    #         iterdata = self.next()
-    #         if self.signature is None or not isinstance(self.signature, Signature):
-    #             self.signature = Signature()
-    #             self.signature.name = 'data_provider'
-    #         if isinstance(arg_names, (list, tuple)) and len(iterdata) == len(arg_names):
-    #             new_dict = OrderedDict()
-    #             for i in range(len(arg_names)):
-    #                 arg = arg_names[i]
-    #                 data = iterdata[i]
-    #                 self.signature.outputs[arg] = TensorSpec(shape=TensorShape((None,) + tuple(tensor_to_shape(data).dims)), dtype=data.dtype,
-    #                                                          object_type=self.datasets_dict.value_list[i].object_type, name=arg)
-    #                 ds = self.datasets_dict.value_list[i]
-    #                 new_dict[arg] = ds
-    #             self.datasets_dict = None
-    #             self.datasets_dict = new_dict
 
     def paired_transform(self, datadict: Dict[TensorSpec, np.ndarray]):
-        # if isinstance(img_data, list) and all(isinstance(elem, np.ndarray) for elem in img_data):
-        #     img_data = np.asarray(img_data)
-        # if isinstance(img_data, str) and os.path.isfile(img_data) and os.path.exists(img_data):
-        #     img_data = image2array(img_data)
-
         if len(self.paired_transform_funcs) == 0:
             return datadict
 
@@ -1035,19 +1178,19 @@ class Iterator(object):
         if self._data and isinstance(self._data, Dataset) and not isinstance(self._data, ZipDataset) and len(self._data) > 0:
             datasets.append(self._data)
         elif self._data and isinstance(self._data, ZipDataset):
-            for ds in self._data._datasets:
+            for ds in self._data.items:
                 datasets.append(ds)
         if self._label and isinstance(self._label, Dataset) and not isinstance(self._label, ZipDataset) and len(self._label) > 0:
             datasets.append(self._label)
         elif self._label and isinstance(self._label, ZipDataset):
-            for ds in self._label._datasets:
+            for ds in self._label.items:
                 datasets.append(ds)
         if self._unpair and isinstance(self._unpair, Dataset) and len(self._unpair) > 0:
             datasets.append(self._unpair)
         elif isinstance(self._unpair, ZipDataset):
-            for ds in self._unpair._datasets:
+            for ds in self._unpair.items:
                 datasets.append(ds)
-        return tuple(datasets)
+        return datasets
 
     def find_dataset(self, dataset_symbol: str):
         datasets = self.get_datasets()
@@ -1055,14 +1198,23 @@ class Iterator(object):
             if ds.symbol == dataset_symbol:
                 return ds
 
-    def update_data_template(self):
-        self._signature=None
+    def update_signature(self):
+        datasets = self.get_datasets()
+        results=self[0]
+        self._signature = Signature(name='data_provider')
+
+        for ds, result,value in zip(datasets, results.key_list,results.value_list):
+            spec = TensorSpec.tensor_to_spec(value, object_type=ds.object_type,need_exclude_batch_axis=True,is_singleton=True, name=ds.symbol)
+            self._signature.outputs[ds.symbol] = spec
+        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffe, drop_last=False)
+        self._sample_iter = iter(self.batch_sampler)
+        return self._signature
 
 
 
     def print_statistics(self):
         print('avg. process time: {0:.5f}'.format(self.pass_time_spend / float(builtins.max(1, self.pass_cnt))))
-        print('avg. batch time: {0:.5f}'.format(self._minibatch_size * self.pass_time_spend / float(builtins.max(1, self.pass_cnt))))
+        print('avg. batch time: {0:.5f}'.format(self._batch_size * self.pass_time_spend / float(builtins.max(1, self.pass_cnt))))
 
     def __getitem__(self, index: int):
         start_time = time.time()
@@ -1071,26 +1223,32 @@ class Iterator(object):
             bbox = None
             mask = None
 
-            returnData = copy.deepcopy(self.data_template)
+            returnData =OrderedDict()# copy.deepcopy(self.data_template)
 
             data = self.data.__getitem__(index % len(self.data)) if self.data is not None and len(self.data) > 0 else None
             label = self.label.__getitem__(index % len(self.label)) if self.label is not None and len(self.label) > 0 else None
 
             unpair = self.unpair.__getitem__(index % len(self.unpair)) if self.unpair is not None and len(self.unpair) > 0 else None
 
-            results = iteration_tools.flatten([data, label, unpair], iterable_types=(list, tuple))
+
+
+            results = iteration_tools.flatten((data, label, unpair), iterable_types=(tuple))
             results = tuple([item for item in results if item is not None])
-            if len(returnData) != len(results):
+
+            if  len(returnData) >0 and  len(returnData) != len(results):
                 raise ValueError("Flattened data sh")
-            for k in range(len(returnData)):
-                returnData[returnData.key_list[k]] = results[k]
+
+            for n  in range(len(self.get_datasets())):
+                ds=self.get_datasets()[n]
+                spec= ds.element_spec
+                returnData[spec] = results[n]
 
             if len(self.paired_process_symbols) > 0:
                 returnData = self.paired_transform(returnData)
 
             def process_data_transform(i):
-                ds = self.datasets_dict[returnData.key_list[i].name]
-                returnData[returnData.key_list[i]] = unpack_singleton(ds.data_transform(returnData.value_list[i]))
+                ds = self.get_datasets()[i]
+                returnData[ds.element_spec] = unpack_singleton(ds.data_transform(returnData.value_list[i]))
 
             threads = []
             for i in range(len(returnData)):
@@ -1108,7 +1266,8 @@ class Iterator(object):
             self.pass_time_spend += float(time.time() - start_time)
 
             return returnData
-        except:
+        except Exception as e:
+            print(e)
             PrintException()
 
     def __setattr__(self, name: str, value) -> None:
@@ -1164,8 +1323,8 @@ class MetricIterator(Iterator):
 
     """
 
-    def __init__(self, data=None, label=None, minibatch_size=8):
-        super().__init__(data, minibatch_size)
+    def __init__(self, data=None, label=None, batch_size=8):
+        super().__init__(data, batch_size)
         self.is_paired_process = False
         self.signature = None
         self._data = ImageDataset()
@@ -1183,9 +1342,9 @@ class MetricIterator(Iterator):
         self.pass_cnt = 0
         self.pass_time_spend = 0
 
-        self._minibatch_size = minibatch_size
+        self._batch_size = batch_size
         self.paired_transform_funcs = []
-        self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=True, drop_last=False)
+        self.batch_sampler = BatchSampler(self, self._batch_size,is_shuffle=self.is_shuffe, drop_last=False)
         self._sample_iter = iter(self.batch_sampler)
         self.buffer_size = 10
         self.out_queue = Queue.Queue(maxsize=self.buffer_size)
