@@ -3,6 +3,7 @@ import math
 import numbers
 import random
 import inspect
+from collections import Iterable
 from functools import wraps
 from typing import Sequence, Tuple, Dict, Union, Optional
 import collections
@@ -27,7 +28,7 @@ from trident.data.transform import VisionTransform
 __all__ = ['Resize', 'ShortestEdgeResize', 'Rescale','RandomCrop','RandomRescaleCrop','RandomCenterCrop','RandomMultiScaleImage','RandomTransform','RandomTransformAffine',
            'AdjustBrightness','AdjustContrast','AdjustSaturation','AddNoise','AdjustHue','RandomAdjustHue','RandomAdjustBrightness','RandomAdjustContrast','RandomAdjustSaturation',
            'Normalize','Unnormalize','CLAHE','Lighting','HorizontalFlip','RandomMirror','AdjustGamma','RandomBlur','RandomAdjustGamma','Blur','InvertColor','RandomInvertColor','GrayScale','RandomGrayScale',
-           'ImageDilation','ImageErosion','ErosionThenDilation','DilationThenErosion','AdaptiveBinarization','SaltPepper','RandomErasing','ToRGB']
+           'ImageDilation','ImageErosion','ErosionThenDilation','DilationThenErosion','AdaptiveBinarization','SaltPepperNoise','RandomErasing','ToRGB','ImageMosaic']
 
 
 
@@ -146,15 +147,24 @@ class Resize(VisionTransform):
         h, w, th, tw,pad_vert,pad_horz = self._shape_info
 
         if not self.keep_aspect:
-            return cv2.resize(image, (tw,th), interpolation=self.interpolation)
+            return cv2.resize(image.copy(), (tw,th), interpolation=self.interpolation)
         else:
 
-            image=cv2.resize(image, (tw,th), interpolation=self.interpolation)
-            output=np.zeros((*self.output_size,1 if spec is not None and spec.object_type==ObjectType.gray else 3))
+            resized_image=cv2.resize(image.copy(), (tw,th), interpolation=self.interpolation)
+            shp=list(int_shape(resized_image))
+            shp[:2] = self.output_size
+
+            output=np.zeros(shp)
             if self.align_corner:
-                output[:th,:tw,:]=image
+                if ndim(resized_image)==2 :
+                    output[:th,:tw]=resized_image
+                elif  ndim(resized_image) == 3 :
+                    output[:th, :tw, :] = resized_image
             else:
-                output[pad_vert//2:th+pad_vert, pad_horz//2:tw+pad_horz, :] = image
+                if ndim(resized_image) == 2 :
+                    output[pad_vert//2:th+pad_vert//2, pad_horz//2:tw+pad_horz//2] = resized_image
+                elif ndim(resized_image) == 3 :
+                    output[pad_vert // 2:th + pad_vert // 2, pad_horz // 2:tw + pad_horz // 2,:] = resized_image
             return output
 
     def _apply_coords(self, coords,spec:TensorSpec):
@@ -193,16 +203,16 @@ class Resize(VisionTransform):
                     output[:th, :tw, :] = mask
             else:
                 if mask.ndim == 2:
-                    output[pad_vert // 2:th + pad_vert , pad_horz // 2:tw + pad_horz] = mask
+                    output[pad_vert // 2:th + pad_vert // 2 , pad_horz // 2:tw +pad_horz // 2] = mask
                 elif mask.ndim == 3:
-                    output[pad_vert // 2:th + pad_vert , pad_horz // 2:tw + pad_horz , :] = mask
+                    output[pad_vert // 2:th + pad_vert // 2 , pad_horz // 2:tw +pad_horz // 2 , :] = mask
             mask = np.squeeze(output)
             return mask.astype(mask_dtype)
 
     def _get_shape(self, image):
         if isinstance(self.output_size, int):
             self.output_size = (self.output_size, self.output_size)
-        h, w,c = image.shape
+        h, w = image.shape[:2]
         eh, ew = self.output_size
 
         if not self.keep_aspect:
@@ -218,73 +228,65 @@ class Resize(VisionTransform):
 
 
 class ShortestEdgeResize(VisionTransform):
-    def __init__(
-        self,
-        min_size,
-        max_size,
-        sample_style="range",
-        interpolation=cv2.INTER_LANCZOS4,
-        *,
-        order=None
-    ):
-        super().__init__(order)
+    def __init__(self,output_size,keep_aspect=True,interpolation=cv2.INTER_LANCZOS4,name='short_edge_resize',**kwargs):
+        super().__init__(name)
         self.is_spatial = True
-        if sample_style not in ("range", "choice"):
-            raise NotImplementedError(
-                "{} is unsupported sample style".format(sample_style)
-            )
-        self.sample_style = sample_style
-        if isinstance(min_size, int):
-            min_size = (min_size, min_size)
-        self.min_size = min_size
-        self.max_size = max_size
+        self.output_size=output_size
+        self.keep_aspect=keep_aspect
+
         self.interpolation = interpolation
 
-    def apply(self, input: Tuple,spec:TensorSpec):
-
-        return super().apply(input,spec)
+    def apply(self, input: Tuple, spec: TensorSpec):
+        return super().apply(input, spec)
 
     def _apply_image(self, image,spec:TensorSpec):
         if  self._shape_info is None:
             self._shape_info = self._get_shape(image)
-        h, w, th, tw = self._shape_info
-        if h == th and w == tw:
+        h, w, th, tw, eh, ew,offsetx,offsety = self._shape_info
+        if h == eh and w == ew:
             return image
-        return cv2.resize(image, (tw,th), self.interpolation)
+        image=cv2.resize(image, (tw,th), self.interpolation)
+        if ndim(image)==2:
+            return image.copy()[offsety:offsety+eh,offsetx:offsetx+ew]
+        elif ndim(image)==3:
+            return image.copy()[offsety:offsety+eh,offsetx:offsetx+ew,:]
+
 
     def _apply_coords(self, coords,spec:TensorSpec):
-        h, w, th, tw = self._shape_info
-        if h == th and w == tw:
+        h, w, th, tw, eh, ew,offsetx,offsety = self._shape_info
+        if h == eh and w == ew:
             return coords
-        coords[:, 0] = coords[:, 0] * (tw / w)
-        coords[:, 1] = coords[:, 1] * (th / h)
+        coords[:, 0] = clip(coords[:, 0] *self.scale-offsetx,0,ew)
+        coords[:, 1] = clip(coords[:, 1] *self.scale-offsety,0,eh)
         return coords
 
     def _apply_mask(self, mask,spec:TensorSpec):
-        h, w, th, tw = self._shape_info
-        if h == th and w == tw:
+        h, w, th, tw, eh, ew, offsetx, offsety = self._shape_info
+        if h == eh and w == ew:
             return mask
-        return cv2.resize(mask,(tw,th), cv2.INTER_NEAREST)
+        mask = cv2.resize(mask, (tw, th), cv2.INTER_NEAREST)
+        if ndim(mask) == 2:
+            return mask.copy()[offsety:offsety + eh, offsetx:offsetx + ew]
+        elif ndim(mask) == 3:
+            return mask.copy()[offsety:offsety + eh, offsetx:offsetx + ew, :]
 
     def _get_shape(self, image):
+        if isinstance(self.output_size, int):
+            self.output_size = (self.output_size, self.output_size)
         h, w = image.shape[:2]
-        if self.sample_style == "range":
-            size = np.random.randint(self.min_size[0], self.min_size[1] + 1)
-        else:
-            size = np.random.choice(self.min_size)
+        eh,ew=self.output_size
+
+        self.scale = builtins.max(eh/h,ew/w)
+        th = int(builtins.round(h * self.scale, 0))
+        tw = int(builtins.round(w * self.scale, 0))
+
+        offsetx=int(random.randint(0,int(tw-ew)) if tw-ew>=1 else 0)
+        offsety = int(random.randint(0, int(th - eh)) if th-eh>=1 else 0)
+        return h, w, th, tw, eh, ew,offsetx,offsety
+
+
+
 
-        scale = size / min(h, w)
-        if h < w:
-            th, tw = size, scale * w
-        else:
-            th, tw = scale * h, size
-        if max(th, tw) > self.max_size:
-            scale = self.max_size / max(th, tw)
-            th = th * scale
-            tw = tw * scale
-        th = int(round(th))
-        tw = int(round(tw))
-        return h, w, th, tw
 
 
 class Rescale(VisionTransform):
@@ -383,7 +385,7 @@ class RandomRescaleCrop(VisionTransform):
 
 
     def _get_shape(self, image):
-        height, width, _ = image.shape
+        height, width= image.shape[:2]
         area = height * width
         if isinstance(self.output_size, int):
             self.output_size = (self.output_size, self.output_size)
@@ -427,11 +429,15 @@ class RandomCenterCrop(VisionTransform):
         if  self._shape_info is None:
             self._shape_info = self._get_shape(image)
         x, y, th, tw, eh, ew,h,w = self._shape_info
-        image=cv2.resize(image, (tw,th), interpolation=self.interpolation)
-        crop_image = image[y: builtins.min(y + eh, th), x:builtins.min( x + ew,tw)]
+        resized_image=cv2.resize(image.copy(), (tw,th), interpolation=self.interpolation)
+        crop_image = resized_image[y: builtins.min(y + eh, th), x:builtins.min( x + ew,tw)]
         if crop_image.shape[0]<eh or crop_image.shape[1]<ew:
             background=np.zeros((eh,ew,1 if spec is not None and spec.object_type==ObjectType.gray else 3))
-            background[builtins.max(eh-crop_image.shape[0],0)//2:builtins.max(eh-crop_image.shape[0],0)//2+crop_image.shape[0],builtins.max(ew-crop_image.shape[1],0)//2:builtins.max(ew-crop_image.shape[1],0)//2+crop_image.shape[1],:]=crop_image
+            if ndim(crop_image)==2:
+                background[builtins.max(eh - crop_image.shape[0], 0) // 2:builtins.max(eh - crop_image.shape[0], 0) // 2 + crop_image.shape[0],
+                builtins.max(ew - crop_image.shape[1], 0) // 2:builtins.max(ew - crop_image.shape[1], 0) // 2 + crop_image.shape[1], 0] = crop_image
+            else:
+                background[builtins.max(eh-crop_image.shape[0],0)//2:builtins.max(eh-crop_image.shape[0],0)//2+crop_image.shape[0],builtins.max(ew-crop_image.shape[1],0)//2:builtins.max(ew-crop_image.shape[1],0)//2+crop_image.shape[1],:]=crop_image
             return background
         else:
             return crop_image
@@ -459,7 +465,7 @@ class RandomCenterCrop(VisionTransform):
 
 
     def _get_shape(self, image):
-        h, w, _ = image.shape
+        h, w = image.shape[:2]
 
         if isinstance(self.output_size, int):
             self.output_size = (self.output_size, self.output_size)
@@ -498,13 +504,18 @@ class RandomCrop(VisionTransform):
         if  self._shape_info is None:
             self._shape_info = self._get_shape(image)
         h,w,eh, ew,offset_x,offset_y,offset_x1,offset_y1 = self._shape_info
-        if image.ndim == 2:
+        if image.ndim == 2 or (image.ndim == 3 and int_shape(image)[-1]==1):
+            origin_ndim=image.ndim
+            if origin_ndim==3:
+                image=image[:,:,0]
             output = np.zeros(self.output_size)
             crop_im = image[offset_y:min(offset_y + eh,h), offset_x:min(offset_x + ew,w)]
             output[offset_y1:offset_y1 + crop_im.shape[0], offset_x1:offset_x1 + crop_im.shape[1]] = crop_im
+            if origin_ndim == 3:
+                output=np.expand_dims(output,-1)
             return output
         elif image.ndim == 3:
-            output=np.zeros((*self.output_size,1 if spec is not None and spec.object_type==ObjectType.gray else 3))
+            output=np.zeros(self.output_size+(1,) if spec is not None and spec.object_type==ObjectType.gray else self.output_size+(3,))
             crop_im = image[offset_y:min(offset_y + eh,h), offset_x:min(offset_x + ew,w),:]
             output[offset_y1:offset_y1 + crop_im.shape[0], offset_x1:offset_x1 + crop_im.shape[1],:] = crop_im
             return output
@@ -530,12 +541,11 @@ class RandomCrop(VisionTransform):
             return output
 
     def _get_shape(self, image):
-        h, w, _ = image.shape
+        h, w= image.shape[:2]
         if isinstance(self.output_size, int):
             self.output_size = (self.output_size, self.output_size)
         eh, ew = self.output_size
 
-
         offset_x = 0
         offset_y = 0
 
@@ -712,9 +722,10 @@ RandomTransform=RandomTransformAffine
 
 
 class RandomMultiScaleImage(VisionTransform):
-    def __init__(self, output_size,scale_range=(0.8, 1.2), interpolation=cv2.INTER_LANCZOS4,name='random_center_crop',**kwargs):
+    def __init__(self, output_size,scale_range=(0.8, 1.2), interpolation=cv2.INTER_LANCZOS4,keep_aspect=True,name='random_multiscale_image',**kwargs):
         super().__init__(name)
         self.is_spatial = True
+        self.keep_aspect=keep_aspect
         self.output_size = output_size
         self.scale_range=scale_range
         self.interpolation = interpolation
@@ -722,13 +733,19 @@ class RandomMultiScaleImage(VisionTransform):
         self.resize_funs=[Resize(output_size,True,align_corner=True,interpolation=interpolation),
                           Resize(output_size, True,align_corner=False, interpolation=interpolation),
                           Resize(output_size, False, interpolation=interpolation),
+                          ShortestEdgeResize(output_size=output_size,keep_aspect=True,interpolation=interpolation),
+                          ShortestEdgeResize(output_size=output_size, keep_aspect=True, interpolation=interpolation),
                           RandomRescaleCrop(output_size=output_size,scale_range=scale_range,interpolation=interpolation),
+                          RandomRescaleCrop(output_size=output_size, scale_range=((scale_range[0]+1)/2,(scale_range[1]+1)/2), interpolation=interpolation),
+                          RandomCrop(output_size=output_size),
                           RandomCrop(output_size=output_size),
                           RandomCenterCrop(output_size=output_size,scale_range=scale_range,interpolation=interpolation)]
+        if self.keep_aspect:
+            self.resize_funs.pop(2)
+
 
     def __call__(self, inputs: Union[Dict[TensorSpec, np.ndarray], np.ndarray], **kwargs):
-        idx =random.randint(0,5)
-        fn = self.resize_funs[idx]
+        fn = random_choice(self.resize_funs)
         spec = kwargs.get('spec')
         return fn.apply_batch(inputs, spec)
 
@@ -894,12 +911,12 @@ class AddNoise(VisionTransform):
 
     def _apply_image(self, image,spec:TensorSpec):
         rr = random.randint(0, 10)
-        orig_min = image.min()
-        orig_max = image.max()
-        noise = np.random.standard_normal(image.shape) * (self.intensity * (orig_max - orig_min))
+        orig_mean = image.mean()
+        orig_std = np.std(image)
+        noise = self.intensity*np.random.normal(0,orig_std,image.shape)
         if rr % 2 == 0:
-            noise = np.random.uniform(-1, 1, image.shape) * (self.intensity * (orig_max - orig_min))
-        image = np.clip(image + noise, orig_min, orig_max)
+            noise =  self.intensity*np.random.uniform(orig_mean-orig_std, orig_mean+orig_std, image.shape)
+        image = np.clip(image + noise,0, 255)
         return image
 
     def _apply_coords(self, coords,spec:TensorSpec):
@@ -1181,10 +1198,13 @@ class ToRGB(VisionTransform):
         return self._apply_image(input,spec)
 
     def _apply_image(self, image,spec:TensorSpec):
+        if  image.ndim == 3 and int_shape(image)[-1]==1:
+            image=image.copy()[:,:,0]
         if image.ndim == 3:
-            return image
+            pass
         elif image.ndim == 2:
-            return cv2.cvtColor(image.astype(np.float32), cv2.COLOR_GRAY2RGB)
+            image=cv2.cvtColor(image.astype(np.float32), cv2.COLOR_GRAY2RGB)
+        return image
 
     def _apply_coords(self, coords,spec:TensorSpec):
         return coords
@@ -1474,7 +1494,6 @@ class Lighting(VisionTransform):
 
 class CLAHE(VisionTransform):
 
-
     def __init__(self, clipLimit=5,gridsize=8,name='clahe',**kwargs):
         super().__init__(name)
         self.gridsize=gridsize
@@ -1500,7 +1519,7 @@ class CLAHE(VisionTransform):
         return mask
 
 
-class SaltPepper(VisionTransform):
+class SaltPepperNoise(VisionTransform):
     """Perform gamma correction on an image.
         Also known as Power Law Transform. Intensities in RGB mode are adjusted
         based on the following equation:
@@ -1513,21 +1532,26 @@ class SaltPepper(VisionTransform):
         gain (float): The constant multiplier.
     """
 
-    def __init__(self, prob=0.01, name='adaptive_binarization', **kwargs):
+    def __init__(self, prob=0.005,keep_prob=0.5, name='saltpepper', **kwargs):
         super().__init__(name)
         self.prob=prob
+        self.keep_prob=keep_prob
 
 
     def apply(self, input: Tuple,spec:TensorSpec):
         return super().apply(input,spec)
 
     def _apply_image(self, image,spec:TensorSpec):
-        imgtype = image.dtype
-        rnd = np.random.rand(image.shape[0], image.shape[1])
-        #noisy = image.copy()
-        image[rnd < self.prob / 2] = 0.0
-        image[rnd > 1 - self.prob / 2] = 255.0
-        return clip(image,0,255).astype(np.float32)
+        rr=random.random()
+        if rr>self.keep_prob:
+            imgtype = image.dtype
+            rnd = np.random.rand(image.shape[0], image.shape[1])
+            #noisy = image.copy()
+            image[rnd < self.prob / 2] = 0.0
+            image[rnd > 1 - self.prob / 2] = 255.0
+            return clip(image,0,255).astype(np.float32)
+        else:
+            return image
 
     def _apply_coords(self, coords,spec:TensorSpec):
         return coords
@@ -1549,54 +1573,223 @@ class RandomErasing(VisionTransform):
         gain (float): The constant multiplier.
     """
 
-    def __init__(self, size_range=(0.05,0.4),transparency_range=(0.4,0.8),transparancy_ratio=0.5, name='random_erasing', **kwargs):
+    def __init__(self, size_range=(0.05,0.4),transparency_range=(0.4,0.8),transparancy_ratio=0.5, keep_prob=0.5,name='random_erasing', **kwargs):
         super().__init__(name)
         self.size_range=size_range
         self.transparency_range=transparency_range
         self.transparancy_ratio=transparancy_ratio
+        self.keep_prob=keep_prob
 
 
     def apply(self, input: Tuple,spec:TensorSpec):
         return super().apply(input,spec)
 
     def _apply_image(self, image,spec:TensorSpec):
-        s_l, s_h = self.size_range
-        r_1 = 0.3
-        r_2 = 1 / 0.3
-        h, w, c = image.shape
-        p_1 = np.random.rand()
+        rr = random.random()
+        if rr > self.keep_prob:
+            s_l, s_h = self.size_range
+            r_1 = 0.3
+            r_2 = 1 / 0.3
+            h, w, c = image.shape
+            p_1 = np.random.rand()
 
-        if p_1 > 0.5:
-            return image
+            if p_1 > 0.5:
+                return image
 
-        while True:
-            s = np.random.uniform(s_l, s_h) * h * w / 4.0
-            r = np.random.uniform(r_1, r_2)
-            w1 = int(np.sqrt(s / r))
-            h1 = int(np.sqrt(s * r))
-            left = np.random.randint(0, w)
-            top = np.random.randint(0, h)
+            while True:
+                s = np.random.uniform(s_l, s_h) * h * w / 4.0
+                r = np.random.uniform(r_1, r_2)
+                w1 = int(np.sqrt(s / r))
+                h1 = int(np.sqrt(s * r))
+                left = np.random.randint(0, w)
+                top = np.random.randint(0, h)
 
-            if left + w1 <= w and top + h1 <= h:
-                break
-        self.rr = np.random.uniform(0, 1)
-        if self.rr  <= self.transparancy_ratio:
-            transparancy = np.random.uniform(*self.transparency_range)
-            mask = np.ones_like(image)
-            mask[top:top + h1, left:left + w1, :] = 0
-            image = image * (mask) + image * (1 - mask) * (transparancy)
-        else:
-
-            if self.rr % 2 == 1:
-                c1 = np.random.uniform(0, 255, (h1, w1, c))
+                if left + w1 <= w and top + h1 <= h:
+                    break
+            self.rr = np.random.uniform(0, 1)
+            if self.rr  <= self.transparancy_ratio:
+                transparancy = np.random.uniform(*self.transparency_range)
+                mask = np.ones_like(image)
+                mask[top:top + h1, left:left + w1, :] = 0
+                image = image * (mask) + image * (1 - mask) * (transparancy)
             else:
-                c1 = np.random.uniform(0, 255)
 
-            image[top:top + h1, left:left + w1, :] = c1
-        return clip(image,0,255).astype(np.float32)
+                if self.rr % 2 == 1:
+                    c1 = np.random.uniform(0, 255, (h1, w1, c))
+                else:
+                    c1 = np.random.uniform(0, 255)
+
+                image[top:top + h1, left:left + w1, :] = c1
+            return clip(image,0,255).astype(np.float32)
+        else:
+            return image
 
     def _apply_coords(self, coords,spec:TensorSpec):
         return coords
 
     def _apply_mask(self, mask,spec:TensorSpec):
         return mask
+
+
+class ImageMosaic(VisionTransform):
+    """Perform gamma correction on an image.
+        Also known as Power Law Transform. Intensities in RGB mode are adjusted
+        based on the following equation:
+        I_out = 255 * gain * ((I_in / 255) ** gamma)
+        See https://en.wikipedia.org/wiki/Gamma_correction for more details.
+    Args:
+        gamma (float): Non negative real number. gamma larger than 1 make the
+            shadows darker, while gamma smaller than 1 make dark regions
+            lighter.
+        gain (float): The constant multiplier.
+    """
+
+    def __init__(self,output_size, keep_prob=0.7,name='image_mosaic', **kwargs):
+        super().__init__(name)
+        self.output_size=output_size
+        self.keep_prob=keep_prob
+
+
+    def apply_batch(self, inputs: Sequence[Tuple],spec:Optional[TensorSpec]=None):
+        r"""Apply transform on batch input data."""
+        if not isinstance(inputs,OrderedDict) :
+            if spec is None and self.is_spatial==True:
+                self._shape_info =None
+                spec = TensorSpec(shape=tensor_to_shape(inputs,need_exclude_batch_axis=True,is_singleton=True), object_type=ObjectType.rgb)
+            return self.apply(inputs, spec)
+        else:
+            results=OrderedDict()
+            imgs=None
+            boxes=None
+            masks=None
+            imgs = inputs.value_list[0]
+
+            if int_shape(imgs)[0]<=4:
+                return inputs
+            else:
+                for i in range(len(inputs.values())):
+                    k,v=inputs.item_list[i]
+                    results[k] = []
+                    if i==0:
+                        imgs=v
+                    elif  isinstance(k,TensorSpec) and 'box' in str(k.object_type).lower():
+                            boxes =v
+                    elif  isinstance(k,TensorSpec) and 'mask' in str(k.object_type).lower():
+                            masks =v
+                if get_backend() == 'pytorch':
+                    imgs = imgs.transpose([0, 2, 3, 1])
+
+                for i in range(int_shape(imgs)[0]):
+                    rr=random.random()
+                    if rr>self.keep_prob:
+
+                        img=imgs[i]
+
+                        other_idxes=list(range(int_shape(imgs)[0]))
+                        other_idxes.remove(i)
+                        random.shuffle(other_idxes)
+                        other_idxes=other_idxes[:3]
+
+                        height,width =self.output_size
+                        rs = np.random.uniform(0.5, 1.5, [2])  # random shift
+                        center = (int(height * rs[0] / 2), int(width * rs[1] / 2))
+
+                        # crop each image
+                        p1= imgs[other_idxes[0]].copy()[:center[0],center[1]:width,:]
+                        p2 =imgs[other_idxes[1]].copy()[center[0]:height,:center[1],:]
+                        p3=imgs[other_idxes[2]].copy()[center[0]:height,center[1]:width,:]
+
+                        base_img=img.copy()
+                        base_img[:center[0],center[1]:width,:]=p1
+                        base_img[center[0]:height,:center[1],:] = p2
+                        base_img[center[0]:height,center[1]:width,:] = p3
+
+                        results[results.key_list[0]].append(base_img)
+
+                        if masks is not None  and masks[i].any():
+                            msk=masks[i]
+                            mp1 = masks[other_idxes[0]].copy()[ :center[0], center[1]:width]
+                            mp2 = masks[other_idxes[1]].copy()[center[0]:height, :center[1]]
+                            mp3 = masks[other_idxes[2]].copy()[center[0]:height, center[1]:width]
+
+                            base_msk = msk.copy()
+                            base_msk[ :center[0], center[1]:width] = mp1
+                            base_msk[center[0]:height, :center[1]] = mp2
+                            base_msk[ center[0]:height, center[1]:width] = mp3
+
+                            mask_key=[k for k in results.key_list if 'mask' in str(k.object_type).lower() ]
+                            if mask_key:
+                                results[mask_key[0]].append(base_msk)
+
+                        mosaic_bboxes=[]
+                        if boxes is not None  and boxes[i].any():
+                            box=boxes[i]
+                            box1 = boxes[other_idxes[0]]
+                            box2 =boxes[other_idxes[1]]
+                            box3 =boxes[other_idxes[2]]
+                            box_list=[]
+                            if box.any():
+                                box[:, 0][box[:, 0] > center[1]] = center[1]
+                                box[:, 2][box[:, 2] > center[1]] = center[1]
+                                box[:, 1][box[:, 1] > center[0]] = center[0]
+                                box[:, 3][box[:, 3] > center[0]] = center[0]
+                                box_list.append(box)
+                            if box1.any():
+                                box1[:, 0][box1[:, 0] < center[1]] = center[1]
+                                box1[:, 2][box1[:, 2] < center[1]] = center[1]
+                                box1[:, 1][box1[:, 1] > center[0]] = center[0]
+                                box1[:, 3][box1[:, 3] > center[0]] = center[0]
+                                box_list.append(box1)
+
+                            if box2.any():
+                                box2[:, 0][box2[:, 0] > center[1]] = center[1]
+                                box2[:, 2][box2[:, 2] > center[1]] = center[1]
+                                box2[:, 1][box2[:, 1] < center[0]] = center[0]
+                                box2[:, 3][box2[:, 3] < center[0]] = center[0]
+                                box_list.append(box2)
+
+
+                            if box3.any():
+                                box3[:, 0][box3[:, 0] < center[1]] = center[1]
+                                box3[:, 2][box3[:, 2] < center[1]] = center[1]
+                                box3[:, 1][box3[:, 1] < center[0]] = center[0]
+                                box3[:, 3][box3[:, 3] < center[0]] = center[0]
+                                box_list.append(box3)
+
+
+                            if len(box_list)>0:
+                                mosaic_bboxes = np.concatenate(box_list)
+                                # remove no area bbox
+                                keep_x = mosaic_bboxes[:, 2] - mosaic_bboxes[:, 0] > 16
+                                keep_y = mosaic_bboxes[:, 3] - mosaic_bboxes[:, 1] > 16
+                                keep_mask = np.logical_and(keep_x, keep_y)
+                                mosaic_bboxes = mosaic_bboxes[keep_mask]
+                            else:
+                                mosaic_bboxes = list()
+                        box_key = [k for k in results.key_list if 'box' in str(k.object_type).lower()]
+                        if box_key:
+                            results[box_key[0]].append(mosaic_bboxes)
+                    else:
+                        results[results.key_list[0]].append(imgs[i])
+                        mask_key = [k for k in results.key_list if 'mask' in str(k.object_type).lower()]
+                        if mask_key:
+                            results[mask_key[0]].append(masks[i])
+                        box_key = [k for k in results.key_list if 'box' in str(k.object_type).lower()]
+                        if box_key:
+                            results[box_key[0]].append(boxes[i])
+
+            for i in range(len(results)):
+                if i==0:
+                    result_imgs=np.stack(results.value_list[i],0)
+                    if get_backend() == 'pytorch':
+                        result_imgs = result_imgs.transpose([0, 3, 1, 2])
+
+                    results[results.key_list[0]] =result_imgs
+                elif not 'box' in str(results.key_list[i].object_type).lower():
+                    results[results.key_list[i]]=np.array(results.value_list[i])
+            return results
+
+
+    def apply(self, input: Tuple,spec:TensorSpec):
+        return super().apply(input,spec)
+
