@@ -19,7 +19,8 @@ import torch.nn as nn
 import torch.nn.functional as F
 from torch._six import container_abcs
 from torch.nn import init
-
+from trident.models.pretrained_utils import _make_recovery_model_include_top
+from trident.data.vision_transforms import Resize,Normalize
 from trident.backend.common import *
 from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, summary, fix_layer, load,get_device
 from trident.data.image_common import *
@@ -86,7 +87,7 @@ def get_block(Bottleneck, out_channel, num_units, strides=2,keep_filter=True):
 
 
 
-def SEResNet_IR(num_layers=50,Bottleneck=BottleNeck_IR_SE,drop_ratio=0.4,feature_dim=128,input_shape=(3,112,112)):
+def SEResNet_IR(include_top=True,num_layers=50,Bottleneck=BottleNeck_IR_SE,drop_ratio=0.4,feature_dim=128,input_shape=(3,112,112)):
     blocks=OrderedDict()
     blocks['input_layer']=Conv2d_Block((3,3),64,strides=1,auto_pad=True,use_bias=False,activation=PRelu(64),normalization='batch',name='input_layer')
     blocks['body']=Sequential(
@@ -105,30 +106,34 @@ def SEResNet_IR(num_layers=50,Bottleneck=BottleNeck_IR_SE,drop_ratio=0.4,feature
     )
     facenet=Sequential(blocks).to(_device)
     facenet.name=camel2snake('SEResNet_IR')
-    model=ImageClassificationModel(input_shape=input_shape,output=facenet)
-    model.preprocess_flow=[resize((input_shape[1],input_shape[2]),keep_aspect=True),normalize(0,255),normalize([0.485, 0.456, 0.406],[0.229, 0.224, 0.225])]
+    model=FaceRecognitionModel(input_shape=input_shape,output=facenet)
+    model.preprocess_flow=[Resize((input_shape[1],input_shape[2]),keep_aspect=True),Normalize(0,255),Normalize([0.485, 0.456, 0.406],[0.229, 0.224, 0.225])]
     #model.summary()
     return model
 
 
-def SEResNet_IR_50_512(
+def SEResNet_IR_50_512(include_top=True,
              pretrained=True,
              freeze_features=False,
              input_shape=(3,112,112),
+             classes=1000,
              **kwargs):
     if input_shape is not None and len(input_shape)==3:
         input_shape=tuple(input_shape)
     else:
         input_shape=(3, 112, 112)
-    seresnet = SEResNet_IR(num_layers=50,Bottleneck=BottleNeck_IR_SE,drop_ratio=0.4,feature_dim=512,input_shape=input_shape)
+    seresnet = SEResNet_IR(include_top=include_top,num_layers=50,Bottleneck=BottleNeck_IR_SE,drop_ratio=0.4,feature_dim=512,input_shape=input_shape)
     if pretrained == True:
         download_model_from_google_drive('1aLYbFvtvsV2gQ16D_vwzrKdbCgij7IoZ', dirname, 'arcface_se_50_512.pth')
         recovery_model = load(os.path.join(dirname, 'arcface_se_50_512.pth'))
         recovery_model = fix_layer(recovery_model)
         recovery_model.name = 'arcface_se_50_512'
-        recovery_model.eval()
-        recovery_model.to(_device)
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
         seresnet.model = recovery_model
+    else:
+        seresnet.model = _make_recovery_model_include_top(seresnet.model, include_top=include_top, classes=classes, freeze_features=False)
+    seresnet.model.input_shape = input_shape
+    seresnet.model.to(_device)
     return seresnet
 
 
