@@ -36,7 +36,7 @@ from trident.layers.pytorch_normalizations import get_normalization
 from trident.layers.pytorch_pooling import *
 from trident.optims.pytorch_trainer import *
 from trident.models.pytorch_ssd import *
-
+from trident.data.vision_transforms import Resize,Normalize
 __all__ = ['RfbNet', 'generate_priors']
 
 _session = get_session()
@@ -129,7 +129,7 @@ def generate_priors(feature_map_list, shrinkage_list, image_size, min_boxes, cla
                     h = min_box / image_size[1]
                     priors.append([x_center, y_center, w, h])
     print("priors nums:{}".format(len(priors)))
-    priors = to_tensor(priors)  # .view(-1, 4)
+    priors = to_tensor(priors).to(get_device())  # .view(-1, 4)
     if clamp:
         torch.clamp(priors, 0.0, 1.0, out=priors)
     return priors
@@ -228,7 +228,7 @@ def tiny_mobile_rfbnet(filter_base=16, num_classes=2):
 
 
 class RFBnet(Layer):
-    def __init__(self, *args, base_filters=16, num_classes=2, num_regressors=4,prob_threshold=0.7, nms_threshold=0.3, center_variance=0.1, size_variance=0.2,
+    def __init__(self, *args, base_filters=16, num_classes=2, num_regressors=4,detection_threshold=0.7, nms_threshold=0.3, center_variance=0.1, size_variance=0.2,
                  name='tiny_mobile_rfbnet', **kwargs):
         """
 
@@ -242,13 +242,13 @@ class RFBnet(Layer):
         self.backbond1 = Sequential(*backbond[:8], name='backbond1')
         self.backbond2 = Sequential(*backbond[8:11], name='backbond2')
         self.backbond3 = Sequential(*backbond[11:13], name='backbond3')
-        self.prob_threshold = prob_threshold
+        self.detection_threshold = detection_threshold
         self.nms_threshold =nms_threshold
         self.variance = (center_variance, size_variance)
 
         self.num_classes = num_classes
         self.num_regressors = num_regressors
-        self.priors = []
+        self.register_buffer("priors", None)
         self.define_img_size(640)
 
         self.extra = Sequential(Conv2d((1, 1), num_filters=64, strides=1, activation='relu', use_bias=True),
@@ -301,7 +301,7 @@ class RFBnet(Layer):
             for k in range(0, len(feature_map_w_h_list[i])):
                 item_list.append(image_size[i] / feature_map_w_h_list[i][k])
             shrinkage_list.append(item_list)
-        self.priors = generate_priors(feature_map_w_h_list, shrinkage_list, image_size, min_boxes)
+        self.register_buffer("priors",generate_priors(feature_map_w_h_list, shrinkage_list, image_size, min_boxes))
 
     def compute_header(self, i, x):
         confidence = self.classification_headers[i](x)
@@ -377,16 +377,16 @@ class RFBnet(Layer):
 
         return box_scores[picked, :]
 
-    def predict(self, width, height, confidences, boxes, prob_threshold=None, iou_threshold=0.3, top_k=-1):
+    def predict(self, width, height, confidences, boxes, detection_threshold=None, iou_threshold=0.3, top_k=-1):
         boxes = boxes
         confidences = confidences
-        if prob_threshold is not None:
-            self.prob_threshold=prob_threshold
+        if detection_threshold is not None:
+            self.detection_threshold=detection_threshold
         picked_box_probs = []
         picked_labels = []
         for class_index in range(1, confidences.shape[1]):
             probs = confidences[:, class_index]
-            mask = probs > self.prob_threshold
+            mask = probs > self.detection_threshold
             probs = probs[mask]
             if probs.shape[0] == 0:
                 continue
@@ -425,8 +425,8 @@ class RFBnet(Layer):
         pad = np.abs(box1 - box)
         return box[0], pad[0]
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+    def forward(self, x):
+        self.priors.to(self.device)
         confidences = []
         locations = []
 
@@ -472,16 +472,20 @@ def RfbNet(include_top=True,
     else:
         input_shape = (3, 480, 640)
     rfbnet = SsdDetectionModel(input_shape=(3, 480, 640), output=RFBnet(base_filters=base_filters, num_classes=num_classes, num_regressors=num_regressors))
+    rfbnet.detection_threshold=0.7
+    rfbnet.nms_threshold=0.7
     rfbnet.palette[0] = (128, 255, 128)
     rfbnet.palette[1] = (128, 255, 128)
     rfbnet.preprocess_flow = [
-        resize((480, 640), True),
-        normalize(127.5, 127.5)
+        Resize((480, 640), True),
+        Normalize(127.5, 127.5)
     ]
     if pretrained == True:
         download_model_from_google_drive('1T_0VYOHaxoyuG1fAxY-6g0C7pfXiujns', dirname, 'version-RFB-640.pth')
-        recovery_model = load(os.path.join(dirname, 'version-RFB-640.pth'))
-        recovery_model = fix_layer(recovery_model)
+        recovery_model =fix_layer( load(os.path.join(dirname, 'version-RFB-640.pth')))
+        priors=recovery_model.priors.clone()
+        recovery_model.__delattr__("priors")
+        recovery_model.register_buffer("priors",priors)
         recovery_model.name = 'rfb640'
         recovery_model.eval()
         recovery_model.to(_device)
