@@ -30,6 +30,7 @@ from trident.layers.tensorflow_normalizations import get_normalization, BatchNor
 from trident.layers.tensorflow_pooling import *
 from trident.optims.tensorflow_trainer import *
 from trident.backend.tensorflow_serialization import load,save
+from trident.data.vision_transforms import Resize,Normalize
 __all__ = ['DenseNet','DenseNet121','DenseNet161','DenseNet169','DenseNet201','DenseNetFcn']
 
 _session = get_session()
@@ -191,7 +192,7 @@ def DenseNet(blocks,
     with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'imagenet_labels1.txt'), 'r',encoding='utf-8-sig') as f:
         labels = [l.rstrip() for l in f]
         model.class_names = labels
-    model.preprocess_flow = [resize((input_shape[0], input_shape[1]), keep_aspect=True), normalize(0, 255),  normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])]
+    model.preprocess_flow = [Resize((input_shape[0], input_shape[1]), keep_aspect=True), Normalize(0, 255),  Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])]
     # model.summary()
     return model
 
@@ -240,8 +241,8 @@ def DenseNetFcn(blocks=(4, 5, 7, 10, 12),
     if is_tensor(model._output_shape):
         model.signature.outputs['output']=TensorSpec(shape=model._output_shape,name='output')
 
-    model.preprocess_flow = [resize((input_shape[2], input_shape[1]), keep_aspect=True), normalize(0, 255),
-                             normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])]
+    model.preprocess_flow = [Resize((input_shape[2], input_shape[1]), keep_aspect=True), Normalize(0, 255),
+                             Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])]
     # model.summary()
     return model
 
